#
# Unit Tests for expression generation 
#
#

import os
import sys
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__)))+"/../..")
currdir = dirname(abspath(__file__))+os.sep

import copy
import StringIO

import pyutilib.th as unittest
from coopr.pyomo import *
from coopr.pyomo.base.expr import *
from coopr.pyomo.base.expr import _SumExpression, _ProductExpression, \
     _IntrinsicFunctionExpression, _PowExpression
from coopr.pyomo.base.var import _VarElement
from coopr.pyomo.base.intrinsic_functions import generate_intrinsic_function_expression
from nose.tools import nottest


class Expression_EvaluateNumericConstant(unittest.TestCase):

    def setUp(self):
        # Do we expect arithmetic operations to return expressions?
        self.expectExpression = False
        # Do we expect relational tests to return constant expressions?
        self.expectConstExpression = True

    def tearDown(self):
        pass

    def create(self,val,domain):
        return NumericConstant(None,domain,val)

    @nottest
    def value_test(self, exp, val, expectExpression=None):
        if expectExpression is None:
            expectExpression = self.expectExpression
        self.assertEqual(isinstance(exp,Expression), expectExpression)
        self.assertEqual(exp(), val)

    @nottest
    def relation_test(self, exp, val, expectConstExpression=None):
        if expectConstExpression is None:
            expectConstExpression = self.expectConstExpression
        # This had better be a expression
        self.assertTrue(isinstance(exp, Expression))
        self.assertEqual(exp.is_relational(), True)
        # Check that the expression evaluates correctly
        self.assertEqual(exp(), val)
        # Check that the expression evaluates correctly in a Boolean context
        if expectConstExpression:
            self.assertEqual(bool(exp), val)
            self.assertIsNone(generate_relational_expression.chainedInequality)
        else:
            self.assertEqual(bool(exp), True)
            self.assertIs(exp,generate_relational_expression.chainedInequality)
            generate_relational_expression.chainedInequality = None

    def test_valid_value(self):
        """Check that values can be validated"""
        a=self.create(1.3, Reals)
        self.assertTrue(a._valid_value(a.value))

        b = self.create(1.3, Integers)
        self.assertRaises(ValueError, b._valid_value, b.value)
        self.assertEqual(b._valid_value(b.value, False), False)

    def Xtest_getattr(self):
        """Check that attributes can be retrieved"""
        a=self.create(1.3,Reals)
        self.assertRaises(AttributeError, a.__getattr__, "_x")
        self.assertRaises(AttributeError, a.__getattr__, "x")

    def Xtest_setattr(self):
        """Check that attributes can be set"""
        a=self.create(1.3,Reals)
        try:
          a.x=1
        except AttributeError:
          pass
        else:
          self.fail("test_setattr")
        a._x=1
        tmp=a._x
        a._standard_attr.add("x")
        a.x=1

    def test_lt(self):
        a=self.create(1.3,Reals)
        b=self.create(2.0,Reals)
        self.relation_test(a<b,True)
        self.relation_test(a<a,False)
        self.relation_test(b<a,False)
        self.relation_test(a<2.0,True)
        self.relation_test(a<1.3,False)
        self.relation_test(b<1.3,False)
        self.relation_test(1.3<b,True)
        self.relation_test(1.3<a,False)
        self.relation_test(2.0<a,False)

    def test_gt(self):
        a=self.create(1.3,Reals)
        b=self.create(2.0,Reals)
        self.relation_test(a>b,False)
        self.relation_test(a>a,False)
        self.relation_test(b>a,True)
        self.relation_test(a>2.0,False)
        self.relation_test(a>1.3,False)
        self.relation_test(b>1.3,True)
        self.relation_test(1.3>b,False)
        self.relation_test(1.3>a,False)
        self.relation_test(2.0>a,True)

    def test_eq(self):
        a=self.create(1.3,Reals)
        b=self.create(2.0,Reals)
        self.relation_test(a==b,False,True)
        self.relation_test(a==a,True,True)
        self.relation_test(b==a,False,True)
        self.relation_test(a==2.0,False,True)
        self.relation_test(a==1.3,True,True)
        self.relation_test(b==1.3,False,True)
        self.relation_test(1.3==b,False,True)
        self.relation_test(1.3==a,True,True)
        self.relation_test(2.0==a,False,True)

    def test_arithmetic(self):
        a=self.create(-0.5,Reals)
        b=self.create(2.0,Reals)
        self.value_test(a-b,-2.5)
        self.value_test(a+b,1.5)
        self.value_test(a*b,-1.0)
        self.value_test(b/a,-4.0)
        self.value_test(a**b,0.25,True)

        self.value_test(a-2.0,-2.5)
        self.value_test(a+2.0,1.5)
        self.value_test(a*2.0,-1.0)
        self.value_test(b/(0.5),4.0)
        self.value_test(a**2.0,0.25,True)

        self.value_test(0.5-b,-1.5)
        self.value_test(0.5+b,2.5)
        self.value_test(0.5*b,1.0)
        self.value_test(2.0/a,-4.0)
        self.value_test((0.5)**b,0.25,True)

        self.value_test(-a,0.5)
        self.value_test(+a,-0.5,False)
        self.value_test(abs(-a),0.5,True)

    # FIXME: This doesn't belong here: we need to create a test_numvalue.py
    def test_asnum(self):
        try:
            as_numeric(None)
            self.fail("test_asnum - expected ValueError")
        except ValueError:
            pass


class Expression_EvaluateVarValue(Expression_EvaluateNumericConstant):

    def setUp(self):
        import coopr.pyomo.base.var
        #
        # Create Model
        #
        Expression_EvaluateNumericConstant.setUp(self)
        #
        # Create model instance
        #
        self.expectExpression = True
        self.expectConstExpression = False

    def create(self,val,domain):
        tmp=coopr.pyomo.base.var._VarValue("unknown",domain)
        tmp.value=val
        return tmp


class Expression_EvaluateVar(Expression_EvaluateNumericConstant):

    def setUp(self):
        import coopr.pyomo.base.var
        #
        # Create Model
        #
        Expression_EvaluateNumericConstant.setUp(self)
        #
        # Create model instance
        #
        self.expectExpression = True
        self.expectConstExpression = False

    def create(self,val,domain):
        tmp=Var(name="unknown",domain=domain)
        tmp.value=val
        return tmp


class Expression_EvaluateFixedVar(Expression_EvaluateNumericConstant):

    def setUp(self):
        import coopr.pyomo.base.var
        #
        # Create Model
        #
        Expression_EvaluateNumericConstant.setUp(self)
        #
        # Create model instance
        #
        self.expectExpression = True
        self.expectConstExpression = True

    def create(self,val,domain):
        tmp=Var(name="unknown",domain=domain)
        tmp.fixed=True
        tmp.value=val
        return tmp


class Expression_EvaluateParam(Expression_EvaluateNumericConstant):

    def setUp(self):
        import coopr.pyomo.base.var
        #
        # Create Model
        #
        Expression_EvaluateNumericConstant.setUp(self)
        #
        # Create model instance
        #
        self.expectExpression = True
        self.expectConstExpression = True

    def create(self,val,domain):
        tmp=Param(within=domain)
        tmp.value = val
        return tmp


class TestNumericValue(pyutilib.th.TestCase):

    def test_vals(self):
        # the following aspect of this test is being removed due to the
        # check seminatics of a numeric constant requiring far too much
        # run-time, especially when involved in expression tree
        # construction. if the user specifies a constant, we're assuming
        # it is correct.
    
        #try:
        #    NumericConstant(None,None,value='a')
        #    self.fail("Cannot initialize a constant with a non-numeric value")
        #except ValueError:
        #    pass
        
        a = NumericConstant(None,None,1.1)
        b = float(a)
        self.assertEqual(b,1.1)
        b = int(a)
        self.assertEqual(b,1)

    def Xtest_getattr1(self):
        a = NumericConstant(None,None,1.1)
        try:
            a.model
            self.fail("Expected error")
        except AttributeError:
            pass

    def test_ops(self):
        a = NumericConstant(None,None,1.1)
        b = NumericConstant(None,None,2.2)
        c = NumericConstant(None,None,-2.2)
        a <= b
        self.assertEqual(a() <= b(), True)
        self.assertEqual(a() >= b(), False)
        self.assertEqual(a() == b(), False)
        self.assertEqual(abs(a() + b()-3.3) <= 1e-7, True)
        self.assertEqual(abs(b() - a()-1.1) <= 1e-7, True)
        self.assertEqual(abs(b() * 3-6.6) <= 1e-7, True)
        self.assertEqual(abs(b() / 2-1.1) <= 1e-7, True)
        self.assertEqual(abs(abs(-b())-2.2) <= 1e-7, True)
        self.assertEqual(abs(c()), 2.2)
        self.assertEqual(str(c), "-2.2")



class Generate_SumExpression(pyutilib.th.TestCase):
    def test_simpleSum(self):
        m = AbstractModel()
        m.a = Var()
        m.b = Var()
        e = m.a + m.b
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 0)
        self.assertEqual(len(e._args), 2)
        self.assertIs(e._args[0], m.a)
        self.assertIs(e._args[1], m.b)
        self.assertEqual(len(e._coef), 2)
        self.assertEqual(e._coef[0], 1)
        self.assertEqual(e._coef[1], 1)
        
    def test_constSum(self):
        m = AbstractModel()
        m.a = Var()
        e = m.a + 5
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 5)
        self.assertEqual(len(e._args), 1)
        self.assertIs(e._args[0], m.a)
        self.assertEqual(len(e._coef), 1)
        self.assertEqual(e._coef[0], 1)

        e = 5 + m.a
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 5)
        self.assertEqual(len(e._args), 1)
        self.assertIs(e._args[0], m.a)
        self.assertEqual(len(e._coef), 1)
        self.assertEqual(e._coef[0], 1)

    def test_nestedSum(self):
        m = AbstractModel()
        m.a = Var()
        m.b = Var()
        m.c = Var()
        m.d = Var()
        e1 = m.a + m.b
        e = e1 + 5
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 5)
        self.assertEqual(len(e._args), 2)
        self.assertIs(e._args[0], m.a)
        self.assertIs(e._args[1], m.b)
        self.assertEqual(len(e._coef), 2)
        self.assertEqual(e._coef[0], 1)
        self.assertEqual(e._coef[1], 1)

        e = 5 + e1
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 5)
        self.assertEqual(len(e._args), 2)
        self.assertIs(e._args[0], m.a)
        self.assertIs(e._args[1], m.b)
        self.assertEqual(len(e._coef), 2)
        self.assertEqual(e._coef[0], 1)
        self.assertEqual(e._coef[1], 1)

        e = e1 + m.c
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 0)
        self.assertEqual(len(e._args), 3)
        self.assertIs(e._args[0], m.a)
        self.assertIs(e._args[1], m.b)
        self.assertIs(e._args[2], m.c)
        self.assertEqual(len(e._coef), 3)
        self.assertEqual(e._coef[0], 1)
        self.assertEqual(e._coef[1], 1)
        self.assertEqual(e._coef[1], 1)

        e = m.c + e1
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 0)
        self.assertEqual(len(e._args), 3)
        self.assertIs(e._args[0], m.c)
        self.assertIs(e._args[1], m.a)
        self.assertIs(e._args[2], m.b)
        self.assertEqual(len(e._coef), 3)
        self.assertEqual(e._coef[0], 1)
        self.assertEqual(e._coef[1], 1)
        self.assertEqual(e._coef[2], 1)

        e2 = m.c + m.d
        e = e1 + e2
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 0)
        self.assertEqual(len(e._args), 4)
        self.assertIs(e._args[0], m.a)
        self.assertIs(e._args[1], m.b)
        self.assertIs(e._args[2], m.c)
        self.assertIs(e._args[3], m.d)
        self.assertEqual(len(e._coef), 4)
        self.assertEqual(e._coef[0], 1)
        self.assertEqual(e._coef[1], 1)
        self.assertEqual(e._coef[2], 1)
        self.assertEqual(e._coef[3], 1)

    def test_trivialSum(self):
        m = AbstractModel()
        m.a = Var()
        e = m.a + 0
        self.assertIs(type(e), type(m.a))
        self.assertIs(e, m.a)

        e = 0 + m.a
        self.assertIs(type(e), type(m.a))
        self.assertIs(e, m.a)

    def test_sumOf_nestedTrivialProduct(self):
        m = AbstractModel()
        m.a = Var()
        m.b = Var()
        m.c = Var()
        e1 = m.a * 5
        e = e1 + m.b
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 0)
        self.assertEqual(len(e._args), 2)
        self.assertIs(e._args[0], m.a)
        self.assertIs(e._args[1], m.b)
        self.assertEqual(len(e._coef), 2)
        self.assertEqual(e._coef[0], 5)
        self.assertEqual(e._coef[1], 1)

        e = m.b + e1
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 0)
        self.assertEqual(len(e._args), 2)
        self.assertIs(e._args[0], m.b)
        self.assertIs(e._args[1], m.a)
        self.assertEqual(len(e._coef), 2)
        self.assertEqual(e._coef[0], 1)
        self.assertEqual(e._coef[1], 5)

        e2 = m.b + m.c
        e = e1 + e2
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 0)
        self.assertEqual(len(e._args), 3)
        self.assertIs(e._args[0], m.a)
        self.assertIs(e._args[1], m.b)
        self.assertIs(e._args[2], m.c)
        self.assertEqual(len(e._coef), 3)
        self.assertEqual(e._coef[0], 5)
        self.assertEqual(e._coef[1], 1)
        self.assertEqual(e._coef[2], 1)

        e2 = m.b + m.c
        e = e2 + e1
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 0)
        self.assertEqual(len(e._args), 3)
        self.assertIs(e._args[0], m.b)
        self.assertIs(e._args[1], m.c)
        self.assertIs(e._args[2], m.a)
        self.assertEqual(len(e._coef), 3)
        self.assertEqual(e._coef[0], 1)
        self.assertEqual(e._coef[1], 1)
        self.assertEqual(e._coef[2], 5)


    def test_simpleDiff(self):
        m = AbstractModel()
        m.a = Var()
        m.b = Var()
        e = m.a - m.b
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 0)
        self.assertEqual(len(e._args), 2)
        self.assertIs(e._args[0], m.a)
        self.assertIs(e._args[1], m.b)
        self.assertEqual(len(e._coef), 2)
        self.assertEqual(e._coef[0], 1)
        self.assertEqual(e._coef[1], -1)
        
    def test_constDiff(self):
        m = AbstractModel()
        m.a = Var()
        e = m.a - 5
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, -5)
        self.assertEqual(len(e._args), 1)
        self.assertIs(e._args[0], m.a)
        self.assertEqual(len(e._coef), 1)
        self.assertEqual(e._coef[0], 1)

        e = 5 - m.a
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 5)
        self.assertEqual(len(e._args), 1)
        self.assertIs(e._args[0], m.a)
        self.assertEqual(len(e._coef), 1)
        self.assertEqual(e._coef[0], -1)

    def test_nestedDiff(self):
        m = AbstractModel()
        m.a = Var()
        m.b = Var()
        m.c = Var()
        m.d = Var()
        e1 = m.a - m.b
        e = e1 - 5
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, -5)
        self.assertEqual(len(e._args), 2)
        self.assertIs(e._args[0], m.a)
        self.assertIs(e._args[1], m.b)
        self.assertEqual(len(e._coef), 2)
        self.assertEqual(e._coef[0], 1)
        self.assertEqual(e._coef[1], -1)

        e = 5 - e1
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 5)
        self.assertEqual(len(e._args), 2)
        self.assertIs(e._args[0], m.a)
        self.assertIs(e._args[1], m.b)
        self.assertEqual(len(e._coef), 2)
        self.assertEqual(e._coef[0], -1)
        self.assertEqual(e._coef[1], 1)

        e = e1 - m.c
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 0)
        self.assertEqual(len(e._args), 3)
        self.assertIs(e._args[0], m.a)
        self.assertIs(e._args[1], m.b)
        self.assertIs(e._args[2], m.c)
        self.assertEqual(len(e._coef), 3)
        self.assertEqual(e._coef[0], 1)
        self.assertEqual(e._coef[1], -1)
        self.assertEqual(e._coef[1], -1)

        e = m.c - e1
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 0)
        self.assertEqual(len(e._args), 3)
        self.assertIs(e._args[0], m.c)
        self.assertIs(e._args[1], m.a)
        self.assertIs(e._args[2], m.b)
        self.assertEqual(len(e._coef), 3)
        self.assertEqual(e._coef[0], 1)
        self.assertEqual(e._coef[1], -1)
        self.assertEqual(e._coef[2], 1)

        e2 = m.c - m.d
        e = e1 - e2
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 0)
        self.assertEqual(len(e._args), 4)
        self.assertIs(e._args[0], m.a)
        self.assertIs(e._args[1], m.b)
        self.assertIs(e._args[2], m.c)
        self.assertIs(e._args[3], m.d)
        self.assertEqual(len(e._coef), 4)
        self.assertEqual(e._coef[0], 1)
        self.assertEqual(e._coef[1], -1)
        self.assertEqual(e._coef[2], -1)
        self.assertEqual(e._coef[3], 1)

        e = e2 - e1
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 0)
        self.assertEqual(len(e._args), 4)
        self.assertIs(e._args[0], m.c)
        self.assertIs(e._args[1], m.d)
        self.assertIs(e._args[2], m.a)
        self.assertIs(e._args[3], m.b)
        self.assertEqual(len(e._coef), 4)
        self.assertEqual(e._coef[0], 1)
        self.assertEqual(e._coef[1], -1)
        self.assertEqual(e._coef[2], -1)
        self.assertEqual(e._coef[3], 1)


    def test_trivialDiff(self):
        m = AbstractModel()
        m.a = Var()
        e = m.a - 0
        self.assertIs(type(e), type(m.a))
        self.assertIs(e, m.a)

        e = 0 - m.a
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 0)
        self.assertEqual(len(e._args), 1)
        self.assertIs(e._args[0], m.a)
        self.assertEqual(len(e._coef), 1)
        self.assertEqual(e._coef[0], -1)

    def test_sumOf_nestedTrivialProduct(self):
        m = AbstractModel()
        m.a = Var()
        m.b = Var()
        m.c = Var()
        e1 = m.a * 5
        e = e1 - m.b
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 0)
        self.assertEqual(len(e._args), 2)
        self.assertIs(e._args[0], m.a)
        self.assertIs(e._args[1], m.b)
        self.assertEqual(len(e._coef), 2)
        self.assertEqual(e._coef[0], 5)
        self.assertEqual(e._coef[1], -1)

        e = m.b - e1
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 0)
        self.assertEqual(len(e._args), 2)
        self.assertIs(e._args[0], m.b)
        self.assertIs(e._args[1], m.a)
        self.assertEqual(len(e._coef), 2)
        self.assertEqual(e._coef[0], 1)
        self.assertEqual(e._coef[1], -5)

        e2 = m.b - m.c
        e = e1 - e2
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 0)
        self.assertEqual(len(e._args), 3)
        self.assertIs(e._args[0], m.a)
        self.assertIs(e._args[1], m.b)
        self.assertIs(e._args[2], m.c)
        self.assertEqual(len(e._coef), 3)
        self.assertEqual(e._coef[0], 5)
        self.assertEqual(e._coef[1], -1)
        self.assertEqual(e._coef[2], 1)

        e = e2 - e1
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 0)
        self.assertEqual(len(e._args), 3)
        self.assertIs(e._args[0], m.b)
        self.assertIs(e._args[1], m.c)
        self.assertIs(e._args[2], m.a)
        self.assertEqual(len(e._coef), 3)
        self.assertEqual(e._coef[0], 1)
        self.assertEqual(e._coef[1], -1)
        self.assertEqual(e._coef[2], -5)

        
class Generate_ProductExpression(pyutilib.th.TestCase):
    def test_simpleProduct(self):
        m = AbstractModel()
        m.a = Var()
        m.b = Var()
        e = m.a * m.b
        self.assertIs(type(e), _ProductExpression)
        self.assertIsNone(e._args)
        self.assertEqual(e.coef, 1)
        self.assertEqual(len(e._numerator), 2)
        self.assertEqual(len(e._denominator), 0)
        self.assertIs(e._numerator[0], m.a)
        self.assertIs(e._numerator[1], m.b)
        
    def test_constProduct(self):
        m = AbstractModel()
        m.a = Var()
        e = m.a * 5
        self.assertIs(type(e), _ProductExpression)
        self.assertIsNone(e._args)
        self.assertEqual(e.coef, 5)
        self.assertEqual(len(e._numerator), 1)
        self.assertEqual(len(e._denominator), 0)
        self.assertIs(e._numerator[0], m.a)

        e = 5 * m.a
        self.assertIs(type(e), _ProductExpression)
        self.assertIsNone(e._args)
        self.assertEqual(e.coef, 5)
        self.assertEqual(len(e._numerator), 1)
        self.assertEqual(len(e._denominator), 0)
        self.assertIs(e._numerator[0], m.a)

    def test_nestedProduct(self):
        m = AbstractModel()
        m.a = Var()
        m.b = Var()
        m.c = Var()
        m.d = Var()
        e1 = m.a * m.b
        e = e1 * 5
        self.assertIs(type(e), _ProductExpression)
        self.assertIsNone(e._args)
        self.assertEqual(e.coef, 5)
        self.assertEqual(len(e._numerator), 2)
        self.assertEqual(len(e._denominator), 0)
        self.assertIs(e._numerator[0], m.a)
        self.assertIs(e._numerator[1], m.b)

        e = 5 * e1
        self.assertIs(type(e), _ProductExpression)
        self.assertIsNone(e._args)
        self.assertEqual(e.coef, 5)
        self.assertEqual(len(e._numerator), 2)
        self.assertEqual(len(e._denominator), 0)
        self.assertIs(e._numerator[0], m.a)
        self.assertIs(e._numerator[1], m.b)

        e = e1 * m.c
        self.assertIs(type(e), _ProductExpression)
        self.assertIsNone(e._args)
        self.assertEqual(e.coef, 1)
        self.assertEqual(len(e._numerator), 3)
        self.assertEqual(len(e._denominator), 0)
        self.assertIs(e._numerator[0], m.a)
        self.assertIs(e._numerator[1], m.b)
        self.assertIs(e._numerator[2], m.c)

        e = m.c * e1
        self.assertIs(type(e), _ProductExpression)
        self.assertIsNone(e._args)
        self.assertEqual(e.coef, 1)
        self.assertEqual(len(e._numerator), 3)
        self.assertEqual(len(e._denominator), 0)
        self.assertIs(e._numerator[0], m.c)
        self.assertIs(e._numerator[1], m.a)
        self.assertIs(e._numerator[2], m.b)

        e2 = m.c * m.d
        e = e1 * e2
        self.assertIs(type(e), _ProductExpression)
        self.assertIsNone(e._args)
        self.assertEqual(e.coef, 1)
        self.assertEqual(len(e._numerator), 4)
        self.assertEqual(len(e._denominator), 0)
        self.assertIs(e._numerator[0], m.a)
        self.assertIs(e._numerator[1], m.b)
        self.assertIs(e._numerator[2], m.c)
        self.assertIs(e._numerator[3], m.d)

    def test_trivialProduct(self):
        m = AbstractModel()
        m.a = Var()
        e = m.a * 0
        self.assertIs(type(e), NumericConstant)
        self.assertEqual(e(), 0)

        e = 0 * m.a
        self.assertIs(type(e), NumericConstant)
        self.assertEqual(e(), 0)

        e = m.a * 1
        self.assertIs(type(e), type(m.a))
        self.assertIs(e, m.a)

        e = 1 * m.a
        self.assertIs(type(e), type(m.a))
        self.assertIs(e, m.a)

        e = NumericConstant(None,None,3) * NumericConstant(None,None,2)
        self.assertIs(type(e), NumericConstant)
        self.assertEqual(e(), 6)


    def test_simpleDivision(self):
        m = AbstractModel()
        m.a = Var()
        m.b = Var()
        e = m.a / m.b
        self.assertIs(type(e), _ProductExpression)
        self.assertIsNone(e._args)
        self.assertEqual(e.coef, 1)
        self.assertEqual(len(e._numerator), 1)
        self.assertEqual(len(e._denominator), 1)
        self.assertIs(e._numerator[0], m.a)
        self.assertIs(e._denominator[0], m.b)
        
    def test_constDivision(self):
        m = AbstractModel()
        m.a = Var()
        e = m.a / 5
        self.assertIs(type(e), _ProductExpression)
        self.assertIsNone(e._args)
        self.assertEqual(e.coef, 1./5.)
        self.assertEqual(len(e._numerator), 1)
        self.assertEqual(len(e._denominator), 0)
        self.assertIs(e._numerator[0], m.a)

        e = 5 / m.a
        self.assertIs(type(e), _ProductExpression)
        self.assertIsNone(e._args)
        self.assertEqual(e.coef, 5)
        self.assertEqual(len(e._numerator), 0)
        self.assertEqual(len(e._denominator), 1)
        self.assertIs(e._denominator[0], m.a)

    def test_nestedDivision(self):
        m = AbstractModel()
        m.a = Var()
        m.b = Var()
        m.c = Var()
        m.d = Var()
        e1 = m.a / m.b
        e = e1 / 5
        self.assertIs(type(e), _ProductExpression)
        self.assertIsNone(e._args)
        self.assertEqual(e.coef, 1./5.)
        self.assertEqual(len(e._numerator), 1)
        self.assertEqual(len(e._denominator), 1)
        self.assertIs(e._numerator[0], m.a)
        self.assertIs(e._denominator[0], m.b)

        e = 5 / e1
        self.assertIs(type(e), _ProductExpression)
        self.assertIsNone(e._args)
        self.assertEqual(e.coef, 5)
        self.assertEqual(len(e._numerator), 1)
        self.assertEqual(len(e._denominator), 1)
        self.assertIs(e._numerator[0], m.b)
        self.assertIs(e._denominator[0], m.a)

        e = e1 / m.c
        self.assertIs(type(e), _ProductExpression)
        self.assertIsNone(e._args)
        self.assertEqual(e.coef, 1)
        self.assertEqual(len(e._numerator), 1)
        self.assertEqual(len(e._denominator), 2)
        self.assertIs(e._numerator[0], m.a)
        self.assertIs(e._denominator[0], m.b)
        self.assertIs(e._denominator[1], m.c)

        e = m.c / e1
        self.assertIs(type(e), _ProductExpression)
        self.assertIsNone(e._args)
        self.assertEqual(e.coef, 1)
        self.assertEqual(len(e._numerator), 2)
        self.assertEqual(len(e._denominator), 1)
        self.assertIs(e._numerator[0], m.c)
        self.assertIs(e._numerator[1], m.b)
        self.assertIs(e._denominator[0], m.a)

        e2 = m.c / m.d
        e = e1 / e2
        self.assertIs(type(e), _ProductExpression)
        self.assertIsNone(e._args)
        self.assertEqual(e.coef, 1)
        self.assertEqual(len(e._numerator), 2)
        self.assertEqual(len(e._denominator), 2)
        self.assertIs(e._numerator[0], m.a)
        self.assertIs(e._numerator[1], m.d)
        self.assertIs(e._denominator[0], m.b)
        self.assertIs(e._denominator[1], m.c)

    def test_trivialDivision(self):
        m = AbstractModel()
        m.a = Var()
        self.assertRaises(ZeroDivisionError, m.a.__div__, 0)

        e = 0 / m.a
        self.assertIs(type(e), NumericConstant)
        self.assertIs(e(), 0)

        e = m.a / 1
        self.assertIs(type(e), type(m.a))
        self.assertIs(e, m.a)

        e = 1 / m.a
        self.assertIs(type(e), _ProductExpression)
        self.assertIsNone(e._args)
        self.assertEqual(e.coef, 1)
        self.assertEqual(len(e._numerator), 0)
        self.assertEqual(len(e._denominator), 1)
        self.assertIs(e._denominator[0], m.a)

        e = NumericConstant(None,None,3) / NumericConstant(None,None,2)
        self.assertIs(type(e), NumericConstant)
        self.assertEqual(e(), 1.5)


class Generate_RelationalExpression(pyutilib.th.TestCase):
    def setUp(self):
        m = AbstractModel()
        m.I = Set()
        m.a = Var()
        m.b = Var()
        m.c = Var()
        m.x = Var(m.I)
        self.m = m

    def test_simpleEquality(self):
        m = self.m
        e = m.a == m.b
        self.assertIs(type(e), _EqualityExpression)
        self.assertEqual(len(e._args), 2)
        self.assertIs(e._args[0], m.a)
        self.assertIs(e._args[1], m.b)

    def test_equalityErrors(self):
        m = self.m
        e = m.a == m.b
        # Python 2.7 supports better testing of exceptions
        if sys.hexversion >= 0x02070000:
            self.assertRaisesRegexp(TypeError, "EqualityExpression .*"
                                   "sub-expressions is a relational",
                                   e.__eq__, m.a)
            self.assertRaisesRegexp(TypeError, "EqualityExpression .*"
                                   "sub-expressions is a relational",
                                   m.a.__eq__, e)

            # NB: cannot test the reverse here: _VarArray (correctly)
            # does not define __eq__
            self.assertRaisesRegexp(TypeError, "Argument .*"
                                    "is an indexed numeric value",
                                    m.a.__eq__, m.x)
        else:
            self.assertRaises(TypeError, e.__eq__, m.a)
            self.assertRaises(TypeError, m.a.__eq__, e)
            self.assertRaises(TypeError, m.a.__eq__, m.x)

        try:
            e == m.a
            self.fail("expected nested equality expression to raise TypeError")
        except TypeError:
            pass

        try:
            m.a == e
            self.fail("expected nested equality expression to raise TypeError")
        except TypeError:
            pass

        try:
            m.x == m.a
            self.fail("expected use of indexed variable to raise TypeError")
        except TypeError:
            pass

        try:
            m.a == m.x
            self.fail("expected use of indexed variable to raise TypeError")
        except TypeError:
            pass

    def test_simpleInequality(self):
        m = self.m
        e = m.a < m.b
        self.assertIs(type(e), _InequalityExpression)
        self.assertEqual(len(e._args), 2)
        self.assertIs(e._args[0], m.a)
        self.assertIs(e._args[1], m.b)
        self.assertEqual(len(e._strict), 1)
        self.assertEqual(e._strict[0], True)

        e = m.a <= m.b
        self.assertIs(type(e), _InequalityExpression)
        self.assertEqual(len(e._args), 2)
        self.assertIs(e._args[0], m.a)
        self.assertIs(e._args[1], m.b)
        self.assertEqual(len(e._strict), 1)
        self.assertEqual(e._strict[0], False)

        e = m.a > m.b
        self.assertIs(type(e), _InequalityExpression)
        self.assertEqual(len(e._args), 2)
        self.assertIs(e._args[0], m.b)
        self.assertIs(e._args[1], m.a)
        self.assertEqual(len(e._strict), 1)
        self.assertEqual(e._strict[0], True)

        e = m.a >= m.b
        self.assertIs(type(e), _InequalityExpression)
        self.assertEqual(len(e._args), 2)
        self.assertIs(e._args[0], m.b)
        self.assertIs(e._args[1], m.a)
        self.assertEqual(len(e._strict), 1)
        self.assertEqual(e._strict[0], False)

        
    def test_compoundInequality(self):
        m = self.m
        e = m.a < m.b < m.c
        self.assertIs(type(e), _InequalityExpression)
        self.assertEqual(len(e._args), 3)
        self.assertIs(e._args[0], m.a)
        self.assertIs(e._args[1], m.b)
        self.assertIs(e._args[2], m.c)
        self.assertEqual(len(e._strict), 2)
        self.assertEqual(e._strict[0], True)
        self.assertEqual(e._strict[1], True)

        e = m.a <= m.b <= m.c
        self.assertIs(type(e), _InequalityExpression)
        self.assertEqual(len(e._args), 3)
        self.assertIs(e._args[0], m.a)
        self.assertIs(e._args[1], m.b)
        self.assertIs(e._args[2], m.c)
        self.assertEqual(len(e._strict), 2)
        self.assertEqual(e._strict[0], False)
        self.assertEqual(e._strict[1], False)

        e = m.a > m.b > m.c
        self.assertIs(type(e), _InequalityExpression)
        self.assertEqual(len(e._args), 3)
        self.assertIs(e._args[0], m.c)
        self.assertIs(e._args[1], m.b)
        self.assertIs(e._args[2], m.a)
        self.assertEqual(len(e._strict), 2)
        self.assertEqual(e._strict[0], True)
        self.assertEqual(e._strict[1], True)

        e = m.a >= m.b >= m.c
        self.assertIs(type(e), _InequalityExpression)
        self.assertEqual(len(e._args), 3)
        self.assertIs(e._args[0], m.c)
        self.assertIs(e._args[1], m.b)
        self.assertIs(e._args[2], m.a)
        self.assertEqual(len(e._strict), 2)
        self.assertEqual(e._strict[0], False)
        self.assertEqual(e._strict[1], False)

        e = 0 <= m.a < 1
        self.assertIs(type(e), _InequalityExpression)
        self.assertEqual(len(e._args), 3)
        self.assertEqual(e._args[0](), 0)
        self.assertIs(e._args[1], m.a)
        self.assertEqual(e._args[2](), 1)
        self.assertEqual(len(e._strict), 2)
        self.assertEqual(e._strict[0], False)
        self.assertEqual(e._strict[1], True)

        e = 0 <= 1 < m.a
        self.assertIs(type(e), _InequalityExpression)
        self.assertEqual(len(e._args), 2)
        self.assertEqual(e._args[0](), 1)
        self.assertIs(e._args[1], m.a)
        self.assertEqual(len(e._strict), 1)
        self.assertEqual(e._strict[0], True)

        e = m.a <= 0 <= 1
        self.assertIs(type(e), bool)
        self.assertEqual(e, True)
        self.assertIsNotNone(generate_relational_expression.chainedInequality)
        try:
            m.a == m.b
            self.fail("expected construction of relational expression to "
                      "generate a chainedInequality TypeError")
        except TypeError:
            pass
        
    def test_inequalityErrors(self):
        m = self.m
        e = m.a <= m.b <= m.c
        e1 = m.a == m.b
        # Python 2.7 supports better testing of exceptions
        if sys.hexversion >= 0x02070000:
            self.assertRaisesRegexp(TypeError, "Argument .*"
                                    "is an indexed numeric value",
                                    m.a.__lt__, m.x)
            self.assertRaisesRegexp(TypeError, "Argument .*"
                                    "is an indexed numeric value",
                                    m.a.__gt__, m.x)

            self.assertRaisesRegexp(ValueError, "InequalityExpression .*"
                                   "more than 3 terms",
                                   e.__lt__, m.c)
            self.assertRaisesRegexp(ValueError, "InequalityExpression .*"
                                   "more than 3 terms",
                                   e.__gt__, m.c)

            self.assertRaisesRegexp(TypeError, "InequalityExpression .*"
                                   "both sub-expressions are also relational",
                                   e.__lt__, m.a < m.b)

            self.assertRaisesRegexp(TypeError, "InequalityExpression .*"
                                   "sub-expressions is an equality",
                                   m.a.__lt__, e1)
            self.assertRaisesRegexp(TypeError, "InequalityExpression .*"
                                   "sub-expressions is an equality",
                                   m.a.__gt__, e1)
        else:
            self.assertRaises(TypeError, m.a.__lt__, m.x)
            self.assertRaises(TypeError, m.a.__gt__, m.x)
            self.assertRaises(ValueError, e.__lt__, m.c)
            self.assertRaises(ValueError, e.__gt__, m.c)
            self.assertRaises(TypeError, e.__lt__, m.a < m.b)
            self.assertRaises(TypeError, m.a.__lt__, e1)
            self.assertRaises(TypeError, m.a.__gt__, e1)

        try:
            m.x < m.a
            self.fail("expected use of indexed variable to raise TypeError")
        except TypeError:
            pass

        try:
            m.a < m.x
            self.fail("expected use of indexed variable to raise TypeError")
        except TypeError:
            pass

        try:
            e < m.c
            self.fail("expected 4-term inequality to raise ValueError")
        except ValueError:
            pass

        try:
            m.c < e
            self.fail("expected 4-term inequality to raise ValueError")
        except ValueError:
            pass

        try:
            e1 = m.a < m.b
            e < e1
            self.fail("expected inequality of inequalities to raise TypeError")
        except TypeError:
            pass

        try:
            m.a < (m.a == m.b)
            self.fail("expected equality within inequality to raise TypeError")
        except TypeError:
            pass

        try:
            m.a > (m.a == m.b)
            self.fail("expected equality within inequality to raise TypeError")
        except TypeError:
            pass
        

class PrettyPrinter(pyutilib.th.TestCase):
    def test_sum(self):
        model = ConcreteModel()
        model.a = Var()

        expr = 5 + model.a + model.a
        buf = StringIO.StringIO()
        expr.pprint(ostream=buf)
        self.assertEqual("sum( 5 , a , a ) \n", buf.getvalue())

        expr += 5
        buf = StringIO.StringIO()
        expr.pprint(ostream=buf)
        self.assertEqual("sum( 10 , a , a ) \n", buf.getvalue())

    def test_prod(self):
        model = ConcreteModel()
        model.a = Var()

        expr = 5 * model.a * model.a
        buf = StringIO.StringIO()
        expr.pprint(ostream=buf)
        self.assertEqual("prod( num=( 5 , a , a ) ) \n", buf.getvalue())

        expr = expr*0
        buf = StringIO.StringIO()
        expr.pprint(ostream=buf)
        self.assertEqual("0", buf.getvalue())

        expr = 5 * model.a / model.a
        buf = StringIO.StringIO()
        expr.pprint(ostream=buf)
        self.assertEqual( "prod( num=( 5 , a ) , denom=( a ) ) \n",
                          buf.getvalue() )

        expr = expr / model.a
        buf = StringIO.StringIO()
        expr.pprint(ostream=buf)
        self.assertEqual( "prod( num=( 5 , a ) , denom=( a , a ) ) \n",
                          buf.getvalue() )

    def test_inequality(self):
        model = ConcreteModel()
        model.a = Var()

        expr = 5 < model.a
        buf = StringIO.StringIO()
        expr.pprint(ostream=buf)
        self.assertEqual( "5  <  a \n", buf.getvalue() )
        
        expr = model.a >= 5
        buf = StringIO.StringIO()
        expr.pprint(ostream=buf)
        self.assertEqual( "5  <=  a \n", buf.getvalue() )
        
        expr = expr < 10
        buf = StringIO.StringIO()
        expr.pprint(ostream=buf)
        self.assertEqual( "5  <=  a  <  10 \n", buf.getvalue() )

        expr = 5 <= model.a + 5
        buf = StringIO.StringIO()
        expr.pprint(ostream=buf)
        self.assertEqual( "5  <=  sum( 5 , a ) \n", buf.getvalue() )

        expr = expr < 10
        buf = StringIO.StringIO()
        expr.pprint(ostream=buf)
        self.assertEqual( "5  <=  sum( 5 , a )  <  10 \n", buf.getvalue() )
        
    def test_equality(self):
        model = ConcreteModel()
        model.a = Var()
        model.b = Param(initialize=5)

        expr = model.a == model.b
        buf = StringIO.StringIO()
        expr.pprint(ostream=buf)
        self.assertEqual( "a  ==  b \n", buf.getvalue() )
        
        expr = model.b == model.a
        buf = StringIO.StringIO()
        expr.pprint(ostream=buf)
        self.assertEqual( "b  ==  a \n", buf.getvalue() )

        # NB: since there is no "reverse equality" operator, explicit
        # constants will always show up second.
        expr = 5 == model.a
        buf = StringIO.StringIO()
        expr.pprint(ostream=buf)
        self.assertEqual( "a  ==  5 \n", buf.getvalue() )
        
        expr = model.a == 10
        buf = StringIO.StringIO()
        expr.pprint(ostream=buf)
        self.assertEqual( "a  ==  10 \n", buf.getvalue() )

        expr = 5 == model.a + 5
        buf = StringIO.StringIO()
        expr.pprint(ostream=buf)
        self.assertEqual( "sum( 5 , a )  ==  5 \n", buf.getvalue() )

        expr = model.a + 5 == 5
        buf = StringIO.StringIO()
        expr.pprint(ostream=buf)
        self.assertEqual( "sum( 5 , a )  ==  5 \n", buf.getvalue() )

    def test_small_expression(self):
        model = AbstractModel()
        model.a = Var()
        model.b = Param(initialize=2)
        instance=model.create()
        expr = instance.a+1
        expr = expr-1
        expr = expr*instance.a
        expr = expr/instance.a
        expr = expr**instance.b
        expr = 1-expr
        expr = 1+expr
        expr = 2*expr
        expr = 2/expr
        expr = 2**expr
        expr = - expr
        expr = + expr
        expr = abs(expr)
        OUTPUT=open(currdir+"expr.out","w")
        expr.pprint(ostream=OUTPUT)
        OUTPUT.close()
        self.assertFileEqualsBaseline(currdir+"expr.out",currdir+"expr.txt")

    def test_large_expression(self):
        def c1_rule(model):
            return (1.0,model.b[1],None)
        def c2_rule(model):
            return (None,model.b[1],0.0)
        def c3_rule(model):
            return (0.0,model.b[1],1.0)
        def c4_rule(model):
            return (3.0,model.b[1])
        def c5_rule(i,model):
            return (model.b[i],0.0)

        def c6a_rule(model):
            return 0.0 <= model.c
        def c6b_rule(model):
            return 0.0 < model.c
        def c7a_rule(model):
            return model.c <= 1.0
        def c7b_rule(model):
            return model.c < 1.0
        def c8_rule(model):
            return model.c == 2.0
        def c9a_rule(model):
            return model.A+model.A <= model.c
        def c9b_rule(model):
            return model.A+model.A < model.c
        def c10a_rule(model):
            return model.c <= model.B+model.B
        def c10b_rule(model):
            return model.c < model.B+model.B
        def c11_rule(model):
            return model.c == model.A+model.B
        def c15a_rule(model):
            return model.A <= model.A*model.d
        def c15b_rule(model):
            return model.A < model.A*model.d
        def c16a_rule(model):
            return model.A*model.d <= model.B
        def c16b_rule(model):
            return model.A*model.d < model.B

        def c12_rule(model):
            return model.c == model.d
        def c13a_rule(model):
            return model.c <= model.d
        def c13b_rule(model):
            return model.c < model.d
        def c14a_rule(model):
            return model.c >= model.d
        def c14b_rule(model):
            return model.c > model.d

        def cl_rule(i, model):
            if i > 10:
                return None
            return i* model.c > model.d

        #def c20_rule(model):
            #return model.A > model.c > model.B
        #def c21_rule(model):
            #return model.A > model.c < model.B
        #def c22_rule(model):
            #return model.A < model.c > model.B
        #def c23_rule(model):
            #return model.A < model.c < model.B

        def o2_rule(i,model):
            return model.b[i]
        model=AbstractModel()
        model.a = Set(initialize=[1,2,3])
        model.b = Var(model.a,initialize=1.1,within=PositiveReals)
        model.c = Var(initialize=2.1, within=PositiveReals)
        model.d = Var(initialize=3.1, within=PositiveReals)
        model.e = Var(initialize=4.1, within=PositiveReals)
        model.A = Param(default=-1)
        model.B = Param(default=-2)
        #model.o1 = Objective()
        model.o2 = Objective(model.a,rule=o2_rule)
        model.o3 = Objective(model.a,model.a)
        model.c1 = Constraint(rule=c1_rule)
        model.c2 = Constraint(rule=c2_rule)
        model.c3 = Constraint(rule=c3_rule)
        model.c4 = Constraint(rule=c4_rule)
        model.c5 = Constraint(model.a,rule=c5_rule)

        model.c6a = Constraint(rule=c6a_rule)
        model.c6b = Constraint(rule=c6b_rule)
        model.c7a = Constraint(rule=c7a_rule)
        model.c7b = Constraint(rule=c7b_rule)
        model.c8 = Constraint(rule=c8_rule)
        model.c9a = Constraint(rule=c9a_rule)
        model.c9b = Constraint(rule=c9b_rule)
        model.c10a = Constraint(rule=c10a_rule)
        model.c10b = Constraint(rule=c10b_rule)
        model.c11 = Constraint(rule=c11_rule)
        model.c15a = Constraint(rule=c15a_rule)
        model.c15b = Constraint(rule=c15b_rule)
        model.c16a = Constraint(rule=c16a_rule)
        model.c16b = Constraint(rule=c16b_rule)

        model.c12 = Constraint(rule=c12_rule)
        model.c13a = Constraint(rule=c13a_rule)
        model.c13b = Constraint(rule=c13b_rule)
        model.c14a = Constraint(rule=c14a_rule)
        model.c14b = Constraint(rule=c14b_rule)

        #model.c20 = Constraint(rule=c20_rule)
        #model.c21 = Constraint(rule=c21_rule)
        #model.c22 = Constraint(rule=c22_rule)
        #model.c23 = Constraint(rule=c23_rule)

        model.cl = ConstraintList(rule=cl_rule)

        instance=model.create()
        OUTPUT=open(currdir+"varpprint.out","w")
        instance.pprint(ostream=OUTPUT)
        OUTPUT.close()
        self.assertFileEqualsBaseline( currdir+"varpprint.out",
                                       currdir+"varpprint.txt" )


class InplaceExpressionGeneration(pyutilib.th.TestCase):
    def setUp(self):
        m = AbstractModel()
        m.a = Var()
        m.b = Var()
        self.m = m
        
    def test_iadd(self):
        m = self.m
        x = 0
        
        count = generate_expression.clone_counter
        x += m.a
        self.assertIs(type(x), _VarElement)
        self.assertEqual(generate_expression.clone_counter, count)

        count = generate_expression.clone_counter
        x += m.a
        self.assertIs(type(x), _SumExpression)
        self.assertEqual(len(x._args), 2)
        self.assertEqual(generate_expression.clone_counter, count)

        count = generate_expression.clone_counter
        x += m.a
        self.assertIs(type(x), _SumExpression)
        self.assertEqual(len(x._args), 3)
        self.assertEqual(generate_expression.clone_counter, count)

        # If someone else holds a reference to the expression, we still
        # need to clone it:
        count = generate_expression.clone_counter
        y = x
        x += m.a
        self.assertIs(type(x), _SumExpression)
        self.assertEqual(len(x._args), 4)
        self.assertEqual(len(y._args), 3)
        self.assertEqual(generate_expression.clone_counter, count+1)
        

    def test_isub(self):
        m = self.m
        x = 0
        
        count = generate_expression.clone_counter
        x -= m.a
        self.assertIs(type(x), _SumExpression)
        self.assertEqual(len(x._args), 1)
        self.assertEqual(generate_expression.clone_counter, count)

        count = generate_expression.clone_counter
        x -= m.a
        self.assertIs(type(x), _SumExpression)
        self.assertEqual(len(x._args), 2)
        self.assertEqual(generate_expression.clone_counter, count)

        count = generate_expression.clone_counter
        x -= m.a
        self.assertIs(type(x), _SumExpression)
        self.assertEqual(len(x._args), 3)
        self.assertEqual(generate_expression.clone_counter, count)

        # If someone else holds a reference to the expression, we still
        # need to clone it:
        count = generate_expression.clone_counter
        y = x
        x -= m.a
        self.assertIs(type(x), _SumExpression)
        self.assertEqual(len(x._args), 4)
        self.assertEqual(len(y._args), 3)
        self.assertEqual(generate_expression.clone_counter, count+1)

    def test_imul(self):
        m = self.m
        x = 1
        
        count = generate_expression.clone_counter
        x *= m.a
        self.assertIs(type(x), _VarElement)
        self.assertEqual(generate_expression.clone_counter, count)

        count = generate_expression.clone_counter
        x *= m.a
        self.assertIs(type(x), _ProductExpression)
        self.assertEqual(len(x._numerator), 2)
        self.assertEqual(len(x._denominator), 0)
        self.assertEqual(generate_expression.clone_counter, count)

        count = generate_expression.clone_counter
        x *= m.a
        self.assertIs(type(x), _ProductExpression)
        self.assertEqual(len(x._numerator), 3)
        self.assertEqual(len(x._denominator), 0)
        self.assertEqual(generate_expression.clone_counter, count)

        # If someone else holds a reference to the expression, we still
        # need to clone it:
        count = generate_expression.clone_counter
        y = x
        x *= m.a
        self.assertIs(type(x), _ProductExpression)
        self.assertEqual(len(x._numerator), 4)
        self.assertEqual(len(y._numerator), 3)
        self.assertEqual(generate_expression.clone_counter, count+1)

    def test_idiv(self):
        m = self.m
        x = 1
        
        count = generate_expression.clone_counter
        x /= m.a
        self.assertIs(type(x), _ProductExpression)
        self.assertEqual(len(x._numerator), 0)
        self.assertEqual(len(x._denominator), 1)
        self.assertEqual(generate_expression.clone_counter, count)

        count = generate_expression.clone_counter
        x /= m.a
        self.assertIs(type(x), _ProductExpression)
        self.assertEqual(len(x._numerator), 0)
        self.assertEqual(len(x._denominator), 2)
        self.assertEqual(generate_expression.clone_counter, count)

        count = generate_expression.clone_counter
        x /= m.a
        self.assertIs(type(x), _ProductExpression)
        self.assertEqual(len(x._numerator), 0)
        self.assertEqual(len(x._denominator), 3)
        self.assertEqual(generate_expression.clone_counter, count)

        # If someone else holds a reference to the expression, we still
        # need to clone it:
        count = generate_expression.clone_counter
        y = x
        x /= m.a
        self.assertIs(type(x), _ProductExpression)
        self.assertEqual(len(x._denominator), 4)
        self.assertEqual(len(y._denominator), 3)
        self.assertEqual(generate_expression.clone_counter, count+1)

    def test_ipow(self):
        m = self.m
        x = 1
        
        count = generate_expression.clone_counter
        x **= m.a
        self.assertIs(type(x), _PowExpression)
        self.assertEqual(len(x._args), 2)
        self.assertEqual(x._args[0](), 1)
        self.assertIs(x._args[1], m.a)
        self.assertEqual(generate_expression.clone_counter, count)

        count = generate_expression.clone_counter
        x **= m.b
        self.assertIs(type(x), _PowExpression)
        self.assertEqual(len(x._args), 2)
        self.assertIs(type(x._args[0]), _PowExpression)
        self.assertIs(x._args[1], m.b)
        self.assertEqual(len(x._args), 2)
        self.assertEqual(x._args[0]._args[0](), 1)
        self.assertIs(x._args[0]._args[1], m.a)
        self.assertEqual(generate_expression.clone_counter, count)

        # If someone else holds a reference to the expression, we still
        # need to clone it:
        count = generate_expression.clone_counter
        x = 1 ** m.a
        y = x
        x **= m.b
        self.assertIs(type(y), _PowExpression)
        self.assertEqual(len(y._args), 2)
        self.assertEqual(y._args[0](), 1)
        self.assertIs(y._args[1], m.a)

        self.assertIs(type(x), _PowExpression)
        self.assertEqual(len(x._args), 2)
        self.assertIs(type(x._args[0]), _PowExpression)
        self.assertIs(x._args[1], m.b)
        self.assertEqual(len(x._args), 2)
        self.assertEqual(x._args[0]._args[0](), 1)
        self.assertIs(x._args[0]._args[1], m.a)
        self.assertEqual(generate_expression.clone_counter, count+1)


class GeneralExpressionGeneration(pyutilib.th.TestCase):

    def test_invalidIndexing(self):
        m = AbstractModel()
        m.A = Set()
        m.p = Param(m.A)
        m.x = Var(m.A)
        m.z = Var()

        try:
            m.p * 2
            self.fail("Expected m.p*2 to raise a TypeError")
        except TypeError:
            pass
       
        try:
            m.x * 2
            self.fail("Expected m.x*2 to raise a TypeError")
        except TypeError:
            pass

        try:
            2 * m.p
            self.fail("Expected 2*m.p to raise a TypeError")
        except TypeError:
            pass
       
        try:
            2 * m.x
            self.fail("Expected 2*m.x to raise a TypeError")
        except TypeError:
            pass

        try:
            m.z * m.p
            self.fail("Expected m.z*m.p to raise a TypeError")
        except TypeError:
            pass
       
        try:
            m.z * m.x
            self.fail("Expected m.z*m.x to raise a TypeError")
        except TypeError:
            pass

    def test_negation(self):
        m = AbstractModel()
        m.a = Var()
        m.b = Var()

        e = -m.a
        self.assertIs(type(e), _ProductExpression)
        self.assertEqual(e.coef, -1)
        self.assertEqual(len(e._numerator), 1)
        self.assertEqual(len(e._denominator), 0)
        self.assertIs(e._numerator[0], m.a)

        e1 = m.a - m.b
        e = -e1
        self.assertIs(type(e), _SumExpression)
        self.assertEqual(e._const, 0)
        self.assertEqual(len(e._args), 2)
        self.assertIs(e._args[0], m.a)
        self.assertIs(e._args[1], m.b)
        self.assertEqual(len(e._coef), 2)
        self.assertEqual(e._coef[0], -1)
        self.assertEqual(e._coef[1], 1)

        e1 = m.a * m.b
        e = -e1
        self.assertIs(type(e), _ProductExpression)
        self.assertEqual(e.coef, -1)
        self.assertEqual(len(e._numerator), 2)
        self.assertEqual(len(e._denominator), 0)
        self.assertIs(e._numerator[0], m.a)
        self.assertIs(e._numerator[1], m.b)

        e1 = sin(m.a)
        e = -e1
        self.assertIs(type(e), _ProductExpression)
        self.assertEqual(e.coef, -1)
        self.assertEqual(len(e._numerator), 1)
        self.assertEqual(len(e._denominator), 0)
        self.assertIs(type(e._numerator[0]), _IntrinsicFunctionExpression)

    def test_genericExpression_notImplemented(self):
        m = AbstractModel()
        m.a = Var()
        m.b = Var()
        e = Expression("dummy", 2, [m.a, m.b])
        self.assertRaises(NotImplementedError, e.clone)
        self.assertRaises(NotImplementedError, e)




class ExprConditionalContext(unittest.TestCase):
    def checkCondition(self, expr, expectedValue):
        try:
            if expr:
                if expectedValue != True:
                    self.fail("__nonzero__ returned the wrong condition value")
            else:
                if expectedValue != False:
                    self.fail("__nonzero__ returned the wrong condition value")
            if expectedValue is None:
                self.fail("Expected ValueError because component was undefined")
        except ValueError:
            if expectedValue is not None:
                raise
        generate_relational_expression.chainedInequality = None

    def tearDown(self):
        # Make sure errors here don't bleed over to other tests
        generate_relational_expression.chainedInequality = None

    def test_paramConditional(self):
        model = AbstractModel()
        model.p = Param(initialize=1.0)
        self.checkCondition(model.p > 0, None)
        self.checkCondition(model.p >= 0, None)
        self.checkCondition(model.p < 1, None)
        self.checkCondition(model.p <= 1, None)
        self.checkCondition(model.p == 0, None)

        instance = model.create()
        self.checkCondition(instance.p > 0, True)
        self.checkCondition(instance.p > 2, False)
        self.checkCondition(instance.p >= 1, True)
        self.checkCondition(instance.p >= 2, False)
        self.checkCondition(instance.p < 2, True)
        self.checkCondition(instance.p < 0, False)
        self.checkCondition(instance.p <= 1, True)
        self.checkCondition(instance.p <= 0, False)
        self.checkCondition(instance.p == 1, True)
        self.checkCondition(instance.p == 2, False)

    def test_paramConditional_reversed(self):
        model = AbstractModel()
        model.p = Param(initialize=1.0)
        self.checkCondition(0 < model.p, None)
        self.checkCondition(0 <= model.p, None)
        self.checkCondition(1 > model.p, None)
        self.checkCondition(1 >= model.p, None)
        self.checkCondition(0 == model.p, None)

        instance = model.create()
        self.checkCondition(0 < instance.p, True)
        self.checkCondition(2 < instance.p, False)
        self.checkCondition(1 <= instance.p, True)
        self.checkCondition(2 <= instance.p, False)
        self.checkCondition(2 > instance.p, True)
        self.checkCondition(0 > instance.p, False)
        self.checkCondition(1 >= instance.p, True)
        self.checkCondition(0 >= instance.p, False)
        self.checkCondition(1 == instance.p, True)
        self.checkCondition(2 == instance.p, False)

    def test_varConditional(self):
        model = AbstractModel()
        model.v = Var(initialize=1.0)
        self.checkCondition(model.v > 0, True)
        self.checkCondition(model.v >= 0, True)
        self.checkCondition(model.v < 1, True)
        self.checkCondition(model.v <= 1, True)
        self.checkCondition(model.v == 0, None)

        instance = model.create()
        self.checkCondition(instance.v > 0, True)
        self.checkCondition(instance.v > 2, True)
        self.checkCondition(instance.v >= 1, True)
        self.checkCondition(instance.v >= 2, True)
        self.checkCondition(instance.v < 2, True)
        self.checkCondition(instance.v < 0, True)
        self.checkCondition(instance.v <= 1, True)
        self.checkCondition(instance.v <= 0, True)
        self.checkCondition(instance.v == 1, True)
        self.checkCondition(instance.v == 2, False)

    def test_varConditional_reversed(self):
        model = AbstractModel()
        model.v = Var(initialize=1.0)
        self.checkCondition(0 < model.v, True)
        self.checkCondition(0 <= model.v, True)
        self.checkCondition(1 > model.v, True)
        self.checkCondition(1 >= model.v, True)
        self.checkCondition(0 == model.v, None)

        instance = model.create()
        self.checkCondition(0 < instance.v, True)
        self.checkCondition(2 < instance.v, True)
        self.checkCondition(1 <= instance.v, True)
        self.checkCondition(2 <= instance.v, True)
        self.checkCondition(2 > instance.v, True)
        self.checkCondition(0 > instance.v, True)
        self.checkCondition(1 >= instance.v, True)
        self.checkCondition(0 >= instance.v, True)
        self.checkCondition(1 == instance.v, True)
        self.checkCondition(2 == instance.v, False)

    def test_eval_sub_varConditional(self):
        model = AbstractModel()
        model.v = Var(initialize=1.0)
        try:
            self.checkCondition(value(model.v) > 0, None)
            self.fail("Expected ValueError because component was undefined")
        except ValueError:
            pass
        try:
            self.checkCondition(value(model.v) >= 0, None)
            self.fail("Expected ValueError because component was undefined")
        except ValueError:
            pass
        try:
            self.checkCondition(value(model.v) < 1, None)
            self.fail("Expected ValueError because component was undefined")
        except ValueError:
            pass
        try:
            self.checkCondition(value(model.v) <= 1, None)
            self.fail("Expected ValueError because component was undefined")
        except ValueError:
            pass
        try:
            self.checkCondition(value(model.v) == 0, None)
            self.fail("Expected ValueError because component was undefined")
        except ValueError:
            pass

        instance = model.create()
        self.checkCondition(value(instance.v) > 0, True)
        self.checkCondition(value(instance.v) > 2, False)
        self.checkCondition(value(instance.v) >= 1, True)
        self.checkCondition(value(instance.v) >= 2, False)
        self.checkCondition(value(instance.v) < 2, True)
        self.checkCondition(value(instance.v) < 0, False)
        self.checkCondition(value(instance.v) <= 1, True)
        self.checkCondition(value(instance.v) <= 0, False)
        self.checkCondition(value(instance.v) == 1, True)
        self.checkCondition(value(instance.v) == 2, False)

    def test_eval_sub_varConditional_reversed(self):
        model = AbstractModel()
        model.v = Var(initialize=1.0)
        try:
            self.checkCondition(0 < value(model.v), None)
            self.fail("Expected ValueError because component was undefined")
        except ValueError:
            pass
        try:
            self.checkCondition(0 <= value(model.v), None)
            self.fail("Expected ValueError because component was undefined")
        except ValueError:
            pass
        try:
            self.checkCondition(1 > value(model.v), None)
            self.fail("Expected ValueError because component was undefined")
        except ValueError:
            pass
        try:
            self.checkCondition(1 >= value(model.v), None)
            self.fail("Expected ValueError because component was undefined")
        except ValueError:
            pass
        try:
            self.checkCondition(0 == value(model.v), None)
            self.fail("Expected ValueError because component was undefined")
        except ValueError:
            pass

        instance = model.create()
        self.checkCondition(0 < value(instance.v), True)
        self.checkCondition(2 < value(instance.v), False)
        self.checkCondition(1 <= value(instance.v), True)
        self.checkCondition(2 <= value(instance.v), False)
        self.checkCondition(2 > value(instance.v), True)
        self.checkCondition(0 > value(instance.v), False)
        self.checkCondition(1 >= value(instance.v), True)
        self.checkCondition(0 >= value(instance.v), False)
        self.checkCondition(1 == value(instance.v), True)
        self.checkCondition(2 == value(instance.v), False)

    def test_eval_varConditional(self):
        model = AbstractModel()
        model.v = Var(initialize=1.0)
        try:
            self.checkCondition(value(model.v > 0), None)
            self.fail("Expected ValueError because component was undefined")
        except ValueError:
            pass
        try:
            self.checkCondition(value(model.v >= 0), None)
            self.fail("Expected ValueError because component was undefined")
        except ValueError:
            pass
        try:
            self.checkCondition(value(model.v == 0), None)
            self.fail("Expected ValueError because component was undefined")
        except ValueError:
            pass

        instance = model.create()
        self.checkCondition(value(instance.v > 0), True)
        self.checkCondition(value(instance.v > 2), False)
        self.checkCondition(value(instance.v >= 1), True)
        self.checkCondition(value(instance.v >= 2), False)
        self.checkCondition(value(instance.v == 1), True)
        self.checkCondition(value(instance.v == 2), False)

    def test_eval_varConditional_reversed(self):
        model = AbstractModel()
        model.v = Var(initialize=1.0)
        try:
            self.checkCondition(value(0 < model.v), None)
            self.fail("Expected ValueError because component was undefined")
        except ValueError:
            pass
        try:
            self.checkCondition(value(0 <= model.v), None)
            self.fail("Expected ValueError because component was undefined")
        except ValueError:
            pass
        try:
            self.checkCondition(value(0 == model.v), None)
            self.fail("Expected ValueError because component was undefined")
        except ValueError:
            pass

        instance = model.create()
        self.checkCondition(value(0 < instance.v), True)
        self.checkCondition(value(2 < instance.v), False)
        self.checkCondition(value(1 <= instance.v), True)
        self.checkCondition(value(2 <= instance.v), False)
        self.checkCondition(value(1 == instance.v), True)
        self.checkCondition(value(2 == instance.v), False)


class PolynomialDegree(pyutilib.th.TestCase):

    def setUp(self):
        def d_fn(model):
            return model.c+model.c
        self.model=AbstractModel()
        self.model.a = Var(initialize=1.0)
        self.model.b = Var(initialize=2.0)
        self.model.c = Param(initialize=0)
        self.model.d = Param(initialize=d_fn)
        self.instance= self.model.create()

    def test_param(self):
        self.assertEqual(self.model.d.polynomial_degree(), 0)

    def test_var(self):
        save = self.model.a.fixed
        self.model.a.fixed = False
        self.assertEqual(self.model.a.polynomial_degree(), 1)
        self.model.a.fixed = True
        self.assertEqual(self.model.a.polynomial_degree(), 0)
        self.model.a.fixed = save

    def test_simple_sum(self):
        expr = self.model.c + self.model.d
        self.assertEqual(expr.polynomial_degree(), 0)

        expr = self.model.a + self.model.b
        self.assertEqual(expr.polynomial_degree(), 1)
        self.model.a.fixed = True
        self.assertEqual(expr.polynomial_degree(), 1)
        self.model.a.fixed = False

        expr = self.model.a + self.model.c
        self.assertEqual(expr.polynomial_degree(), 1)
        self.model.a.fixed = True
        self.assertEqual(expr.polynomial_degree(), 0)
        self.model.a.fixed = False

    def test_relational_ops(self):
        expr = self.model.c < self.model.d
        self.assertEqual(expr.polynomial_degree(), 0)

        expr = self.model.a <= self.model.d
        self.assertEqual(expr.polynomial_degree(), 1)

        expr = self.model.a * self.model.a >= self.model.b
        self.assertEqual(expr.polynomial_degree(), 2)
        self.model.a.fixed = True
        self.assertEqual(expr.polynomial_degree(), 1)
        self.model.a.fixed = False

        expr = self.model.a > self.model.a * self.model.b
        self.assertEqual(expr.polynomial_degree(), 2)
        self.model.b.fixed = True
        self.assertEqual(expr.polynomial_degree(), 1)
        self.model.b.fixed = False

    def test_simple_product(self):
        expr = self.model.c * self.model.d
        self.assertEqual(expr.polynomial_degree(), 0)

        expr = self.model.a * self.model.b
        self.assertEqual(expr.polynomial_degree(), 2)

        expr = self.model.a * self.model.c
        self.assertEqual(expr.polynomial_degree(), 1)
        self.model.a.fixed = True
        self.assertEqual(expr.polynomial_degree(), 0)
        self.model.a.fixed = False

        expr = self.model.a / self.model.c
        self.assertEqual(expr.polynomial_degree(), 1)

        expr = self.model.c / self.model.a
        self.assertEqual(expr.polynomial_degree(), None)
        self.model.a.fixed = True
        self.assertEqual(expr.polynomial_degree(), 0)
        self.model.a.fixed = False

    def test_nested_expr(self):
        expr1 = self.model.c * self.model.d
        expr2 = expr1 + expr1
        self.assertEqual(expr2.polynomial_degree(), 0)

        expr1 = self.model.a * self.model.b
        expr2 = expr1 + expr1
        self.assertEqual(expr2.polynomial_degree(), 2)
        self.model.a.fixed = True
        self.assertEqual(expr2.polynomial_degree(), 1)
        self.model.a.fixed = False

        expr1 = self.model.c + self.model.d
        expr2 = expr1 * expr1
        self.assertEqual(expr2.polynomial_degree(), 0)

        expr1 = self.model.a + self.model.b
        expr2 = expr1 * expr1
        self.assertEqual(expr2.polynomial_degree(), 2)
        self.model.a.fixed = True
        self.assertEqual(expr2.polynomial_degree(), 2)
        self.model.b.fixed = True
        self.assertEqual(expr2.polynomial_degree(), 0)
        self.model.b.fixed = False
        self.model.a.fixed = False

    def test_misc_operators(self):
        expr = -(self.model.a * self.model.b)
        self.assertEqual(expr.polynomial_degree(), 2)

    def test_nonpolynomial(self):
        expr = abs(self.model.a * self.model.b)
        self.assertEqual(expr.polynomial_degree(), None)

        expr = self.model.a + self.model.b * abs(self.model.b)
        self.assertEqual(expr.polynomial_degree(), None)

        expr = self.model.a * ( self.model.b + abs(self.model.b) )
        self.assertEqual(expr.polynomial_degree(), None)

        # NB: Currently we do not check for special polynomial cases of
        # the pow() function (arg[0] is polynomial and arg[1] is a
        # non-negative integer)
        expr = pow(self.model.a, 2)
        self.assertEqual(expr.polynomial_degree(), None)



class CloneIfNeeded(pyutilib.th.TestCase):

    def setUp(self):
        def d_fn(model):
            return model.c+model.c
        model=ConcreteModel()
        model.I = Set(initialize=range(4))
        model.J = Set(initialize=range(1))
        model.a = Var()
        model.b = Var(model.I)
        model.c = Param(initialize=1)
        model.d = Param(initialize=d_fn)
        self.model = model
        self.refCount = []

    def tearDown(self):
        generate_expression.clone_if_needed_callback = None
        generate_intrinsic_function_expression.clone_if_needed_callback = None
        generate_relational_expression.clone_if_needed_callback = None

    def trapCloneIfNeeded(self, count):
        self.refCount.append(count)

    def test_operator_UNREFERENCED_EXPR_COUNT(self):
        generate_expression.clone_if_needed_callback = \
            lambda x: self.trapCloneIfNeeded(x)

        expr1 = abs(self.model.a+self.model.a)
        self.assertEqual( self.refCount, [0] )

        expr2 = expr1 + self.model.a
        self.assertEqual( self.refCount, [0,1] )
        
    def test_intrinsic_UNREFERENCED_EXPR_COUNT(self):
        generate_intrinsic_function_expression.clone_if_needed_callback = \
            lambda x: self.trapCloneIfNeeded(x)

        val1 = cos(0)
        self.assertTrue( type(val1) is float )
        self.assertEqual( val1, 1 )
        self.assertEqual( self.refCount, [] )

        expr1 = cos(self.model.a+self.model.a)
        self.assertEqual( self.refCount, [0] )

        expr2 = sin(expr1)
        self.assertEqual( self.refCount, [0,1] )

    def test_relational_UNREFERENCED_EXPR_COUNT(self):
        generate_relational_expression.clone_if_needed_callback = \
            lambda x: self.trapCloneIfNeeded(x)

        expr1 = self.model.c < self.model.a + self.model.b[1]
        self.assertEqual( self.refCount, [0] )

        expr2 = expr1 < 5
        self.assertEqual( self.refCount, [0,1] )


    def test_cloneCount_simple(self):
        # simple expression
        count = generate_expression.clone_counter
        expr = self.model.a * self.model.a
        self.assertEqual(generate_expression.clone_counter, count)

        # expression based on another expression
        count = generate_expression.clone_counter
        expr = expr + self.model.a
        self.assertEqual(generate_expression.clone_counter, count + 1)

    def test_cloneCount_sumVars(self):
        # sum over variable using generators
        count = generate_expression.clone_counter
        expr = sum(self.model.b[i] for i in self.model.I)
        self.assertEqual(generate_expression.clone_counter, count)

        # sum over variable using list comprehension
        count = generate_expression.clone_counter
        expr = sum([self.model.b[i] for i in self.model.I])
        self.assertEqual(generate_expression.clone_counter, count)

    def test_cloneCount_sumExpr_singleTerm(self):
        # sum over expression using generators (single element)
        count = generate_expression.clone_counter
        expr = sum(self.model.b[i]*self.model.b[i] for i in self.model.J)
        self.assertEqual(generate_expression.clone_counter, count)

        # sum over expression using list comprehension (single element)
        count = generate_expression.clone_counter
        expr = sum([self.model.b[i]*self.model.b[i] for i in self.model.J])
        self.assertEqual(generate_expression.clone_counter, count+1)

        # sum over expression using list (single element)
        count = generate_expression.clone_counter
        l = [self.model.b[i]*self.model.b[i] for i in self.model.J]
        expr = sum(l)
        self.assertEqual(generate_expression.clone_counter, count+1)

    def test_cloneCount_sumExpr_multiTerm(self):
        # sum over expression using generators
        count = generate_expression.clone_counter
        expr = sum(self.model.b[i]*self.model.b[i] for i in self.model.I)
        self.assertEqual(generate_expression.clone_counter, count)

        # sum over expression using list comprehension
        count = generate_expression.clone_counter
        expr = sum([self.model.b[i]*self.model.b[i] for i in self.model.I])
        self.assertEqual(generate_expression.clone_counter, count+4)

        # sum over expression using list 
        count = generate_expression.clone_counter
        l = [self.model.b[i]*self.model.b[i] for i in self.model.I]
        expr = sum(l)
        self.assertEqual(generate_expression.clone_counter, count+4)

        # generate a new expression from a complex one
        count = generate_expression.clone_counter
        expr1 = expr + 1
        self.assertEqual(generate_expression.clone_counter, count+1)

    def test_cloneCount_sumExpr_complexExpr(self):
        # sum over complex expression using generators
        count = generate_expression.clone_counter
        expr = sum( value(self.model.c)*(1+self.model.b[i])**2
                    for i in self.model.I )
        self.assertEqual(generate_expression.clone_counter, count)

        # sum over complex expression using list comprehension
        count = generate_expression.clone_counter
        expr = sum([ value(self.model.c)*(1+self.model.b[i])**2 
                     for i in self.model.I ])
        self.assertEqual(generate_expression.clone_counter, count+4)

        # sum over complex expression using list
        count = generate_expression.clone_counter
        l = [ value(self.model.c)*(1+self.model.b[i])**2
              for i in self.model.I ]
        expr = sum(l)
        self.assertEqual(generate_expression.clone_counter, count+4)


    def test_cloneCount_intrinsicFunction(self):
        # intrinsicFunction of a simple expression
        count = generate_intrinsic_function_expression.clone_counter
        expr = log(self.model.c + self.model.a)
        self.assertEqual( generate_intrinsic_function_expression.clone_counter,
                          count )

        # intrinsicFunction of a referenced expression
        count = generate_intrinsic_function_expression.clone_counter
        expr = self.model.c + self.model.a
        expr1 = log(expr)
        self.assertEqual( generate_intrinsic_function_expression.clone_counter,
                          count+1 )


    def test_cloneCount_relationalExpression_simple(self):
        # relational expression of simple vars
        count = generate_relational_expression.clone_counter
        expr = self.model.c < self.model.a
        self.assertEqual(len(expr._args), 2)
        self.assertEqual( generate_relational_expression.clone_counter,
                          count )

        # relational expression of simple expressions
        count = generate_relational_expression.clone_counter
        expr = 2*self.model.c < 2*self.model.a
        self.assertEqual(len(expr._args), 2)
        self.assertEqual( generate_relational_expression.clone_counter,
                          count )

        # relational expression of a referenced expression
        count = generate_relational_expression.clone_counter
        expr = self.model.c + self.model.a
        expr1 = expr < self.model.d
        self.assertEqual(len(expr._args), 2)
        self.assertEqual( generate_relational_expression.clone_counter,
                          count + 1 )

    def test_cloneCount_relationalExpression_compound(self):
        # relational expression of a compound expression (simple vars)
        count = generate_relational_expression.clone_counter
        expr = self.model.c < self.model.a < self.model.d
        expr.pprint()
        self.assertEqual(len(expr._args), 3)
        self.assertEqual( generate_relational_expression.clone_counter,
                          count )

        # relational expression of a compound expression 
        # (non-expression common term)
        count = generate_relational_expression.clone_counter
        expr = 2*self.model.c < self.model.a < 2*self.model.d
        expr.pprint()
        self.assertEqual(len(expr._args), 3)
        self.assertEqual( generate_relational_expression.clone_counter,
                          count )

        # relational expression of a compound expression 
        # (expression common term)
        count = generate_relational_expression.clone_counter
        expr = self.model.c < 2 * self.model.a < self.model.d
        expr.pprint()
        self.assertEqual(len(expr._args), 3)
        self.assertEqual( generate_relational_expression.clone_counter,
                          count + 1 )

        # relational expression of a referenced compound expression (1)
        count = generate_relational_expression.clone_counter
        expr = self.model.c < self.model.a
        expr1 = expr < self.model.d
        expr1.pprint()
        self.assertEqual(len(expr1._args), 3)
        self.assertEqual( generate_relational_expression.clone_counter,
                          count + 1)

        # relational expression of a referenced compound expression (2)
        count = generate_relational_expression.clone_counter
        expr = 2*self.model.c < 2*self.model.a
        expr1 = self.model.d < expr
        expr1.pprint()
        self.assertEqual(len(expr1._args), 3)
        self.assertEqual( generate_relational_expression.clone_counter,
                          count + 1)

    def test_cloneCount_relationalExpression_compound_reversed(self):
        # relational expression of a compound expression (simple vars)
        count = generate_relational_expression.clone_counter
        expr = self.model.c < self.model.a > self.model.d
        expr.pprint()
        self.assertEqual(len(expr._args), 3)
        self.assertEqual( generate_relational_expression.clone_counter,
                          count )

        # relational expression of a compound expression 
        # (non-expression common term)
        count = generate_relational_expression.clone_counter
        expr = 2*self.model.c < self.model.a > 2*self.model.d
        expr.pprint()
        self.assertEqual(len(expr._args), 3)
        self.assertEqual( generate_relational_expression.clone_counter,
                          count )

        # relational expression of a compound expression 
        # (expression common term)
        count = generate_relational_expression.clone_counter
        expr = self.model.c < 2 * self.model.a > self.model.d
        expr.pprint()
        self.assertEqual(len(expr._args), 3)
        self.assertEqual( generate_relational_expression.clone_counter,
                          count + 1 )

        # relational expression of a referenced compound expression (1)
        count = generate_relational_expression.clone_counter
        expr = self.model.c < self.model.a
        expr1 = expr > self.model.d
        expr1.pprint()
        self.assertEqual(len(expr1._args), 3)
        self.assertEqual( generate_relational_expression.clone_counter,
                          count + 1)

        # relational expression of a referenced compound expression (2)
        count = generate_relational_expression.clone_counter
        expr = 2*self.model.c < 2*self.model.a
        expr1 = self.model.d > expr
        expr1.pprint()
        self.assertEqual(len(expr1._args), 3)
        self.assertEqual( generate_relational_expression.clone_counter,
                          count + 1)

class CloneExpression(pyutilib.th.TestCase):
    def setUp(self):
        self.m = ConcreteModel()
        self.m.a = Var(initialize=5)
        self.m.b = Var(initialize=10)
        self.m.expr = self.m.a + self.m.b
    
    def test_IdentityExpression(self):
        expr1 = _IdentityExpression(self.m.a)
        expr2 = expr1.clone()
        self.assertEqual( expr1(), 5 )
        self.assertEqual( expr2(), 5 )
        self.assertNotEqual( id(expr1._args), id(expr2._args) )
        self.assertEqual( id(expr1._args[0]), id(expr2._args[0]) )
        expr1._args = (self.m.b, )
        self.assertEqual( expr1(), 10 )
        self.assertEqual( expr2(), 5 )
        self.assertNotEqual( id(expr1._args), id(expr2._args) )
        self.assertNotEqual( id(expr1._args[0]), id(expr2._args[0]) )

        expr1 = _IdentityExpression(self.m.a + self.m.b)
        expr2 = expr1.clone()
        self.assertEqual( expr1(), 15 )
        self.assertEqual( expr2(), 15 )
        self.assertNotEqual( id(expr1._args), id(expr2._args) )
        self.assertNotEqual( id(expr1._args[0]), id(expr2._args[0]) )
        self.assertEqual( id(expr1._args[0]._args[0]), 
                          id(expr2._args[0]._args[0]) )
        self.assertEqual( id(expr1._args[0]._args[1]), 
                          id(expr2._args[0]._args[1]) )
        expr1._args = (expr1._args[0] + self.m.b,)
        self.assertEqual( expr1(), 25 )
        self.assertEqual( expr2(), 15 )
        self.assertNotEqual( id(expr1._args), id(expr2._args) )
        self.assertNotEqual( id(expr1._args[0]), id(expr2._args[0]) )

    def test_SumExpression(self):
        expr1 = self.m.a + self.m.b
        expr2 = expr1.clone()
        self.assertEqual( expr1(), 15 )
        self.assertEqual( expr2(), 15 )
        self.assertNotEqual( id(expr1._args), id(expr2._args) )
        self.assertEqual( id(expr1._args[0]), id(expr2._args[0]) )
        self.assertEqual( id(expr1._args[1]), id(expr2._args[1]) )
        expr1._args[0] = self.m.b
        self.assertEqual( expr1(), 20 )
        self.assertEqual( expr2(), 15 )
        self.assertNotEqual( id(expr1._args), id(expr2._args) )
        self.assertNotEqual( id(expr1._args[0]), id(expr2._args[0]) )
        self.assertEqual( id(expr1._args[1]), id(expr2._args[1]) )

        expr1 = self.m.a + self.m.b
        expr2 = expr1.clone()
        self.assertEqual( expr1(), 15 )
        self.assertEqual( expr2(), 15 )
        self.assertNotEqual( id(expr1._args), id(expr2._args) )
        self.assertEqual( id(expr1._args[0]), id(expr2._args[0]) )
        self.assertEqual( id(expr1._args[1]), id(expr2._args[1]) )
        expr1 += self.m.b
        self.assertEqual( expr1(), 25 )
        self.assertEqual( expr2(), 15 )
        self.assertNotEqual( id(expr1._args), id(expr2._args) )
        self.assertEqual( id(expr1._args[0]), id(expr2._args[0]) )
        self.assertEqual( id(expr1._args[1]), id(expr2._args[1]) )

    def test_ProductExpression_mult(self):
        expr1 = self.m.a * self.m.b
        expr2 = expr1.clone()
        self.assertEqual( expr1(), 50 )
        self.assertEqual( expr2(), 50 )
        self.assertNotEqual(id(expr1._numerator),    id(expr2._numerator))
        self.assertNotEqual(id(expr1._denominator),  id(expr2._denominator))
        self.assertEqual   (id(expr1._numerator[0]), id(expr2._numerator[0]))
        self.assertEqual   (id(expr1._numerator[1]), id(expr2._numerator[1]))
        expr1._numerator[0] = self.m.b
        self.assertEqual( expr1(), 100 )
        self.assertEqual( expr2(), 50 )
        self.assertNotEqual(id(expr1._numerator),    id(expr2._numerator))
        self.assertNotEqual(id(expr1._denominator),  id(expr2._denominator))
        self.assertNotEqual(id(expr1._numerator[0]), id(expr2._numerator[0]))
        self.assertEqual   (id(expr1._numerator[1]), id(expr2._numerator[1]))

        expr1 = self.m.a * self.m.b
        expr2 = expr1.clone()
        self.assertEqual( expr1(), 50 )
        self.assertEqual( expr2(), 50 )
        self.assertNotEqual(id(expr1._numerator),    id(expr2._numerator))
        self.assertNotEqual(id(expr1._denominator),  id(expr2._denominator))
        self.assertEqual   (id(expr1._numerator[0]), id(expr2._numerator[0]))
        self.assertEqual   (id(expr1._numerator[1]), id(expr2._numerator[1]))
        expr1 *= self.m.b
        self.assertEqual( expr1(), 500 )
        self.assertEqual( expr2(), 50 )
        self.assertNotEqual(id(expr1._numerator),    id(expr2._numerator))
        self.assertNotEqual(id(expr1._denominator),  id(expr2._denominator))
        self.assertEqual   (id(expr1._numerator[0]), id(expr2._numerator[0]))
        self.assertEqual   (id(expr1._numerator[1]), id(expr2._numerator[1]))

    def test_ProductExpression_div(self):
        expr1 = self.m.a / self.m.b
        expr2 = expr1.clone()
        self.assertEqual( expr1(), 0.5 )
        self.assertEqual( expr2(), 0.5 )
        self.assertNotEqual(id(expr1._numerator),   id(expr2._numerator))
        self.assertNotEqual(id(expr1._denominator), id(expr2._denominator))
        self.assertEqual(id(expr1._numerator[0]),   id(expr2._numerator[0]))
        self.assertEqual(id(expr1._denominator[0]), id(expr2._denominator[0]))
        expr1._denominator[0] = self.m.a
        self.assertEqual( expr1(), 1 )
        self.assertEqual( expr2(), 0.5 )
        self.assertNotEqual(id(expr1._numerator),    id(expr2._numerator))
        self.assertNotEqual(id(expr1._denominator),  id(expr2._denominator))
        self.assertEqual   (id(expr1._numerator[0]), id(expr2._numerator[0]))
        self.assertNotEqual( id(expr1._denominator[0]),
                             id(expr2._denominator[0]) )

        expr1 = self.m.a / self.m.b
        expr2 = expr1.clone()
        self.assertEqual( expr1(), 0.5 )
        self.assertEqual( expr2(), 0.5 )
        self.assertNotEqual(id(expr1._numerator),   id(expr2._numerator))
        self.assertNotEqual(id(expr1._denominator), id(expr2._denominator))
        self.assertEqual(id(expr1._numerator[0]),   id(expr2._numerator[0]))
        self.assertEqual(id(expr1._denominator[0]), id(expr2._denominator[0]))
        expr1 /= self.m.b
        self.assertEqual( expr1(), 0.05 )
        self.assertEqual( expr2(), 0.5 )
        self.assertNotEqual(id(expr1._numerator),   id(expr2._numerator))
        self.assertNotEqual(id(expr1._denominator), id(expr2._denominator))
        self.assertEqual(id(expr1._numerator[0]),   id(expr2._numerator[0]))
        self.assertEqual(id(expr1._denominator[0]), id(expr2._denominator[0]))

    def test_sumOfExpressions(self):
        expr1 = self.m.a * self.m.b + self.m.a * self.m.a
        expr2 = expr1.clone()
        self.assertEqual(expr1(), 75) 
        self.assertEqual(expr2(), 75) 
        self.assertNotEqual(id(expr1._args), id(expr2._args))
        self.assertEqual(expr1._args[0](), expr2._args[0]())
        self.assertEqual(expr1._args[1](), expr2._args[1]())
        self.assertNotEqual(id(expr1._args[0]), id(expr2._args[0]))
        self.assertNotEqual(id(expr1._args[1]), id(expr2._args[1]))
        expr1 += self.m.b
        self.assertEqual(expr1(), 85) 
        self.assertEqual(expr2(), 75) 
        self.assertNotEqual(id(expr1._args), id(expr2._args))
        self.assertEqual(len(expr1._args), 3)
        self.assertEqual(len(expr2._args), 2)
        self.assertEqual(expr1._args[0](), expr2._args[0]())
        self.assertEqual(expr1._args[1](), expr2._args[1]())
        self.assertNotEqual(id(expr1._args[0]), id(expr2._args[0]))
        self.assertNotEqual(id(expr1._args[1]), id(expr2._args[1]))
        
    def test_productOfExpressions(self):
        expr1 = (self.m.a + self.m.b) * (self.m.a + self.m.a)
        expr2 = expr1.clone()
        self.assertEqual(expr1(), 150) 
        self.assertEqual(expr2(), 150) 
        self.assertNotEqual(id(expr1._numerator), id(expr2._numerator))
        self.assertNotEqual(id(expr1._denominator), id(expr2._denominator))
        self.assertEqual(expr1._numerator[0](), expr2._numerator[0]())
        self.assertEqual(expr1._numerator[1](), expr2._numerator[1]())
        self.assertNotEqual(id(expr1._numerator[0]), id(expr2._numerator[0]))
        self.assertNotEqual(id(expr1._numerator[1]), id(expr2._numerator[1]))
        expr1 *= self.m.b
        self.assertEqual(expr1(), 1500) 
        self.assertEqual(expr2(), 150) 
        self.assertNotEqual(id(expr1._numerator), id(expr2._numerator))
        self.assertNotEqual(id(expr1._denominator), id(expr2._denominator))
        self.assertEqual(len(expr1._numerator), 3)
        self.assertEqual(len(expr2._numerator), 2)
        self.assertEqual(len(expr1._denominator), 0)
        self.assertEqual(len(expr2._denominator), 0)
        self.assertEqual(expr1._numerator[0](), expr2._numerator[0]())
        self.assertEqual(expr1._numerator[1](), expr2._numerator[1]())
        self.assertNotEqual(id(expr1._numerator[0]), id(expr2._numerator[0]))
        self.assertNotEqual(id(expr1._numerator[1]), id(expr2._numerator[1]))
        
    def test_productOfExpressions_div(self):
        expr1 = (self.m.a + self.m.b) / (self.m.a + self.m.a)
        expr2 = expr1.clone()
        self.assertEqual(expr1(), 1.5) 
        self.assertEqual(expr2(), 1.5) 
        self.assertNotEqual(id(expr1._numerator), id(expr2._numerator))
        self.assertNotEqual(id(expr1._denominator), id(expr2._denominator))
        self.assertEqual(expr1._numerator[0](), expr2._numerator[0]())
        self.assertEqual(expr1._denominator[0](), expr2._denominator[0]())
        self.assertNotEqual(id(expr1._numerator[0]),id(expr2._numerator[0]))
        self.assertNotEqual(id(expr1._denominator[0]),id(expr2._denominator[0]))
        expr1 /= self.m.b
        self.assertEqual(expr1(), .15) 
        self.assertEqual(expr2(), 1.5) 
        self.assertNotEqual(id(expr1._numerator), id(expr2._numerator))
        self.assertNotEqual(id(expr1._denominator), id(expr2._denominator))
        self.assertEqual(len(expr1._numerator), 1)
        self.assertEqual(len(expr2._numerator), 1)
        self.assertEqual(len(expr1._denominator), 2)
        self.assertEqual(len(expr2._denominator), 1)
        self.assertEqual(expr1._numerator[0](), expr2._numerator[0]())
        self.assertEqual(expr1._denominator[0](), expr2._denominator[0]())
        self.assertNotEqual(id(expr1._numerator[0]), id(expr2._numerator[0]))
        self.assertNotEqual(id(expr1._denominator[0]),id(expr2._denominator[0]))
        

if __name__ == "__main__":
   unittest.main()



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['Constraint', 'SOSConstraint', 'ConstraintBase', 'ConstraintList']

import sys
import logging

from expr import *
from indexed_component import IndexedComponent
from numtypes import *
from numvalue import *
from numvalue import create_name
from label import *
from pyutilib.component.core import alias
from sets import _BaseSet, _SetContainer, _ProductSet, Set
from var import Var, _VarValue, _VarArray, _VarElement
from objective import Objective
import pyutilib.math
import pyutilib.misc

from component import Component
from set_types import *
from sets import _SetContainer

logger = logging.getLogger('coopr.pyomo')


class ConstraintBase(IndexedComponent):
    """
    Abstract base class for all constraint types. Keeps track of how many
    constraint objects are in existence.
    """

    alias("ConstraintBase", "Abstract base class for all model constraints")

    # The number of explicit constraints declared.
    _nExplicitConstraints = 0

    def __init__(self, *args, **kwargs):
        ctype = {'ctype': kwargs.pop("ctype", None)}
        IndexedComponent.__init__(self, *args, **ctype)

        tmpname  = kwargs.pop('name', 'unknown')
        self.doc = kwargs.pop('doc', None )

        # Increment the class count
        ConstraintBase._inc_count()

        self._data = {}

    def dim(self):
        return self._ndim

    def __len__(self):
        return len(self._data.keys())

    def keys(self):
        return self._data.keys()
        #return self._index

    def __contains__(self,ndx):
        return ndx in self._data
        #return ndx in self._index

    def __getitem__(self, ndx):
        """This method returns a ConstraintData object.  This object can be
           coerced to a numeric value using the value() function, or using
           explicity coercion with float().
        """
        if ndx in self._data:
            return self._data[ndx]
        msg = "Unknown index in constraint '%s': %s"
        raise KeyError, msg % ( self.name, str(ndx) )

    def __iter__(self):
        return self._data.keys().__iter__()

    def __del__(self):
        """ Remove this constraint from the count """
        self._inc_count(-1)

    @classmethod
    def _inc_count(cls, n=1):
        """ Alter the constraint count """
        cls._nExplicitConstraints += n

    @classmethod
    def num_explicit_constraints(cls):
        return cls._nExplicitConstraints


class ConstraintData(NumericValue):

    __slots__ = ('con','label','id','active','_equality','lower',
                 'lower_ineq_strict','lin_body','body','upper',
                 'upper_ineq_strict','repn','ampl_repn','dual')

    def __init__(self, name, con=None):

        NumericValue.__init__(self,name,Reals,None,True)

        self.con = con

        # the label is a modification of the input name, with certain symbols
        # that are typically problematic for solver input files re-mapped (e.g., 
        # curly braces) to more friendly symbols. create a default label right
        # away - it can be modified if a particular solver needs it.
        # self.label = None (is set one way or another via the following method)
        self.sync_label()

        # TBD: Document where this gets populated.
        self.id = None

        # TBD: Mirroring the variable class, the constraint data class should really "know" its index and its parent constraint.

        # is this variable an active component of the current model?
        self.active = True
        
        self._equality = False
        self.lower = None
        self.lower_ineq_strict = None
        self.lin_body = None # a linear compilation of the source expressions tree.
        self.body = None # an expression tree encoding of the constraint body. if None, an alternative representation (e.g., lin_body) will be != None.
        self.upper = None
        self.upper_ineq_strict = None

        self.repn = None
        self.ampl_repn = None

        self.dual = None

    def __getstate__(self):
       result = NumericValue.__getstate__(self)
       for i in ConstraintData.__slots__:
          result[i] = getattr(self, i)
       return result        

    def sync_label(self):
        if self.name is not None:
            self.label = label_from_name(self.name)
            if (self.con is not None) and (self.con.model is not None):
                self.con.model._name_conmap[self.label] = self
        else:
            self.label = None

    def __call__(self, exception=True):
        if self.body is None:
            return None
        return self.body()

    def activate(self):
        self.active=True

    def deactivate(self):
        self.active=False


class Constraint(ConstraintBase, NumericValue):
    """An object that defines a objective expression"""

    alias("Constraint", "Constraint expressions a model.")

    def __init__(self, *args, **kwargs):
        """
        Construct an objective expression with rule to construct the
        expression

        keyword arguments:
        name: name of this object
        rule: function or rule definition of constraint
        expr: same as rule
        doc:  documentation string for constraint
        """
        # See if 'ctype' was passed as a keyword argument;
        # this allows derived classses to alert Pyomo to
        # their existence through super() calls. If not,
        # pass Constraint
        tkwd = {'ctype': kwargs.pop('ctype', Constraint)}

        # Pass some arguments to ConstraintBase
        tkwd['doc'] = kwargs.pop('doc', None)
        tkwd['name'] = kwargs.get('name', 'unknown')

        ConstraintBase.__init__(self, *args, **tkwd)

        tmpname = kwargs.pop('name', 'unknown')
        tmprule = kwargs.pop('rule', None )
        tmprule = kwargs.pop('expr', tmprule )

        self._no_rule_init = kwargs.pop('noruleinit', None )

        if ( kwargs ): # if dict not empty, there's an error.  Let user know.
            msg = "Creating constraint '%s': unknown option(s)\n\t%s"
            msg = msg % ( tmpname, ', '.join(kwargs.keys()) )
            raise ValueError, msg

        # _no_rule_init is a flag specified by the user to indicate that no
        # construction rule will be specified, and that constraints will be
        # explicitly added by the user. set via the "noruleinit" keyword. value
        # doesn't matter, as long as it isn't "None".

        self._data = {}

        NumericValue.__init__(self,tmpname,None,None,True)        

        if None in self._data:
            # As far as I can tell, this never executes? 2010 Jun 08
            self._data[None].name=tmpname
        self.rule = tmprule
        self.trivial = False # True if all constraint indicies have trivial expressions.

    def __getstate__(self):
       result = NumericValue.__getstate__(self)
       for key,value in self.__dict__.iteritems():
          result[key]=value
       return result            

    def clear(self):
        self._data = {}

    def __call__(self, exception=True):
        if len(self._data) == 0:
            return None
        if None in self._data:
            if self._data[None].body is None:
                return None
            return value(self._data[None].body)
        if exception:
            msg = 'Cannot compute the value of an array of constraints'
            raise ValueError, msg


    def construct(self, data=None):

        # cache the debug generation flag, to avoid the expense of 
        # calling isEnabledFor() for each index - this is far too
        # expensive an operation to perform deep in a loop. the only
        # potential down-side is that the debug logging is either
        # disabled or enabled globally for the duration of this 
        # method invocation - which doesn't seem like much of
        # (if any) limitation in practice.
        generate_debug_messages = (__debug__ is True) and (logger.isEnabledFor(logging.DEBUG) is True)

        if generate_debug_messages is True:
            logger.debug("Constructing constraint %s",self.name)

        if (self._no_rule_init is not None) and (self.rule is not None):
            logger.warn("noruleinit keyword is being used in conjunction with "
                        "rule keyword for constraint '%s'; defaulting to "
                        "rule-based construction.", self.name)
        if self.rule is None:
            if self._no_rule_init is None:
                logger.warn("No construction rule or expression specified for "
                            "constraint '%s'", self.name)
            return
        if self._constructed:
            return
        self._constructed=True

        #
        # Local variables for code optimization
        #
        _self_rule = self.rule
        _self_model = self.model

        #
        if None in self._index:
            if len(self._index) != 1:
                raise IndexError, "Internal error: constructing constraint "\
                    "with both None and Index set"
            if generate_debug_messages:
                logger.debug("  Constructing single constraint (index=None)")
            if isinstance(_self_rule,Expression):
                expr = _self_rule
            else:
                expr = _self_rule(_self_model)
            if expr is not None:
                self.add(None, expr)
        else:
            if isinstance(_self_rule,Expression):
                raise IndexError, "Cannot define multiple indices in a " \
                    "constraint with a single expression"
            for index in self._index:
                if generate_debug_messages:
                    logger.debug("  Constructing constraint index "+str(index))
                if type(index) is tuple:
                    expr = _self_rule(*( index + (_self_model,) ))
                else:
                    expr = _self_rule(index, _self_model)
                if expr is not None:
                    self.add(index, expr)

    def add(self, index, expr):

        # index: the constraint index (should probably be renamed)
        # expr: the constraint expression

        #
        # Ignore an 'empty' constraint
        #
        if expr is None:
            return
        expr_type = type(expr)

        # Local variables to optimize runtime performance
        #
        conData = ConstraintData(name=create_name(self.name,index), con=self)
        conData.lower_ineq_strict = conData.upper_ineq_strict = False

        if expr_type is tuple: # or expr_type is list:
            #
            # Form equality expression
            #
            if len(expr) == 2:
                arg0 = expr[0]
                if arg0 is not None:
                    arg0 = as_numeric(arg0)
                arg1 = expr[1]
                if arg1 is not None:
                    arg1 = as_numeric(arg1)

                conData._equality = True
                if arg1 is None or arg1.is_constant():
                    conData.lower = conData.upper = arg1
                    conData.body = arg0
                elif arg0 is None or arg0.is_constant():
                    conData.lower = conData.upper = arg0
                    conData.body = arg1
                else:
                    conData.lower = conData.upper = NumericConstant(None,None,0)
                    conData.body = arg0 - arg1
            #
            # Form inequality expression
            #
            elif len(expr) == 3:
                arg0 = expr[0]
                if arg0 is not None:
                    arg0 = as_numeric(arg0)
                    if not arg0.is_constant():
                        msg = "Constraint '%s' found a 3-tuple (lower, " \
                              "expression, upper) but the lower value was "\
                              "non-constant"
                        raise ValueError, msg % (conData.name,)

                arg1 = expr[1]
                if arg1 is not None:
                    arg1 = as_numeric(arg1)

                arg2 = expr[2]
                if arg2 is not None:
                    arg2 = as_numeric(arg2)
                    if not arg2.is_constant():
                        msg = "Constraint '%s' found a 3-tuple (lower, " \
                              "expression, upper) but the upper value was "\
                              "non-constant"
                        raise ValueError, msg % (conData.name,)

                conData.lower = arg0
                conData.body  = arg1
                conData.upper = arg2
            else:
                msg = "Constructor rule for constraint '%s' returned a tuple" \
                      ' of length %d.  Expecting a tuple of length 2 or 3:\n' \
                      'Equality:   (left, right)\n' \
                      'Inequality: (lower, expression, upper)'
                raise ValueError, msg % ( self.name, len(expr) )

            relational_expr = False
        else:
            try:
                relational_expr = expr.is_relational()
                if not relational_expr:
                    msg = "Constraint '%s' does not have a proper value.  " \
                          "Found '%s'\nExpecting a tuple or equation.  " \
                          "Examples:\n" \
                          "    summation( model.costs ) == model.income\n" \
                          "    (0, model.price[ item ], 50)"
                    raise ValueError, msg % ( conData.name, str(expr) )
            except:
                msg = "Constraint '%s' does not have a proper value.  " \
                      "Found '%s'\nExpecting a tuple or equation.  " \
                      "Examples:\n" \
                      "    summation( model.costs ) == model.income\n" \
                      "    (0, model.price[ item ], 50)" \
                      % ( conData.name, str(expr) )
                if type(expr) is bool:
                    msg +="""
Note: constant Boolean expressions are not valid constraint expressions.
Some apparently non-constant compound inequalities (e.g. "expr >= 0 <= 1")
can return boolean values; the proper form for compound inequalities is
always "lb <= expr <= ub"."""
                raise ValueError, msg 

        #
        # Special check for chainedInequality errors like "if var < 1:"
        # within rules.  Catching them here allows us to provide the
        # user with better (and more immediate) debugging information.
        # We don't want to check earlier because we want to provide a
        # specific debugging message if the construction rule returned
        # True/False; for example, if the user did ( var < 1 > 0 )
        # (which also results in a non-None chainedInequality value)
        #
        if generate_relational_expression.chainedInequality is not None:
            from expr import chainedInequalityErrorMessage
            raise TypeError, chainedInequalityErrorMessage()

        #
        # Process relational expressions (i.e. explicit '==', '<', and '<=')
        #
        if relational_expr:
            if expr_type is _EqualityExpression:
                # Equality expression: only 2 arguments!
                conData._equality = True
                if expr._args[1].is_constant():
                    conData.lower = conData.upper = expr._args[1]
                    conData.body = expr._args[0]
                elif expr._args[0].is_constant():
                    conData.lower = conData.upper = expr._args[0]
                    conData.body = expr._args[1]
                else:
                    conData.lower = conData.upper = NumericConstant(None,None,0)
                    conData.body = expr._args[0] - expr._args[1]
            else:
                # Inequality expression: 2 or 3 arguments
                if len(expr._args) == 3:
                    if not expr._args[0].is_constant():
                        msg = "Constraint '%s' found a double-sided "\
                              "inequality expression (lower <= expression "\
                              "<= upper) but the lower bound was non-constant"
                        raise ValueError, msg % (conData.name,)
                    if not expr._args[2].is_constant():
                        msg = "Constraint '%s' found a double-sided "\
                              "inequality expression (lower <= expression "\
                              "<= upper) but the upper bound was non-constant"
                        raise ValueError, msg % (conData.name,)
                    conData.lower = expr._args[0]
                    conData.body  = expr._args[1]
                    conData.upper = expr._args[2]
                    conData.lower_ineq_strict = expr._strict[0]
                    conData.upper_ineq_strict = expr._strict[1]
                else:
                    if expr._args[1].is_constant():
                        conData.lower = None
                        conData.body  = expr._args[0]
                        conData.upper = expr._args[1]
                        conData.upper_ineq_strict = expr._strict[0]
                    elif expr._args[0].is_constant():
                        conData.lower = expr._args[0]
                        conData.body  = expr._args[1]
                        conData.upper = None
                        conData.lower_ineq_strict = expr._strict[0]
                    else:
                        conData.lower = None
                        conData.body  = expr._args[0] - expr._args[1]
                        conData.upper = NumericConstant(None,None,0)
                        conData.upper_ineq_strict = expr._strict[0]

        #
        # Replace numeric bound values with a NumericConstant object,
        # and reset the values to 'None' if they are 'infinite'
        #
        if conData.lower is not None:
            val = conData.lower()
            if not pyutilib.math.is_finite(val):
                if val > 0:
                    msg = "Constraint '%s' created with a +Inf lower bound"
                    raise ValueError, msg % ( conData.name, )
                conData.lower = None
                conData.lower_ineq_strict = False
            elif bool(val > 0) == bool(val <= 0):
                msg = "Constraint '%s' created with a non-numeric lower bound"
                raise ValueError, msg % ( conData.name, )
        if conData.upper is not None:
            val = conData.upper()
            if not pyutilib.math.is_finite(val):
                if val < 0:
                    msg = "Constraint '%s' created with a -Inf upper bound"
                    raise ValueError, msg % ( conData.name, )
                conData.upper = None
                conData.upper_ineq_strict = False
            elif bool(val > 0) == bool(val <= 0):
                msg = "Constraint '%s' created with a non-numeric upper bound"
                raise ValueError, msg % ( conData.name, )

        # Error check, to ensure that we don't have a constraint that
        # doesn't depend on any variables / parameters 
        #
        # TBD: This is an expensive test (is_constant() is recursive).
        # The pre-coopr-2.5 version of this test was basically never
        # executed (just looked for int, float, etc.)  It is not clear
        # that running this debugging test is even worth it [JDS, 28 Feb 2011].
        #
        #if conData.body.is_constant():
        #    feasible = True
        #    value = conData.body()
        #    if conData.lower is not None:
        #        if conData.lower_ineq_strict:
        #            feasible = feasible and (conData.lower() < value)
        #        else:
        #            feasible = feasible and (conData.lower() <= value)
        #    if conData.upper is not None:
        #        if conData.upper_ineq_strict:
        #            feasible = feasible and (value < conData.upper())
        #        else:
        #            feasible = feasible and (value < conData.upper())
        #    if feasible:
        #        logger.warn("Constraint '%s' has a constant constant body.")
        #    else:
        #        logger.error("Constraint '%s' has an constant infeasible "
        #                     "constant body.")

        # Error check, to ensure that we don't have an equality constraint with
        # 'infinite' RHS
        #
        if conData._equality:
            if conData.lower != conData.upper: #pragma:nocover
                msg = "Equality constraint '%s' has non-equal lower and "\
                      "upper bounds (this is indicitive of a SERIOUS "\
                      "internal error in Pyomo)."
                raise RuntimeError, msg % self.name
            if conData.lower is None:
                msg = "Equality constraint '%s' defined with non-finite term"
                raise ValueError, msg % self.name

        #
        # hook up the constraint data object to the parent constraint.
        #
        self._data[index] = conData

        #
        # update the parent model (if any) name->constraint map.
        #
        if self.model is not None:
           self.model._name_conmap[conData.label] = conData


    def pprint(self, ostream=None):
        if ostream is None:
           ostream = sys.stdout
        print >>ostream, "  ",self.name,":",
        print >>ostream, "\tSize="+str(len(self._data.keys())),
        if isinstance(self._index,_BaseSet):
           print >>ostream, "\tIndex=",self._index.name
        else:
           print >>ostream,""
        for val in self._data:
          if not val is None:
             print >>ostream, "\t"+`val`
          if self._data[val].lower is not None:
             print >>ostream, "\t\t",
             if self._data[val].lower.is_expression():
                self._data[val].lower.pprint(ostream)
             else:
                print >>ostream, str(self._data[val].lower)
          else:
             print >>ostream, "\t\t-Inf"
          if self._data[val].lower_ineq_strict:
             print >>ostream, "\t\t<"
          else:
             print >>ostream, "\t\t<="
          if self._data[val].body is not None:
             print >>ostream, "\t\t",
             if self._data[val].body.is_expression():
                self._data[val].body.pprint(ostream)
             else:
                print >>ostream, str(self._data[val].body)
          #else:                         #pragma:nocover
             #raise ValueError, "Unexpected empty constraint body"
          if self._data[val].upper_ineq_strict:
             print >>ostream, "\t\t<"
          else:
             print >>ostream, "\t\t<="
          if self._data[val].upper is not None:
             print >>ostream, "\t\t",
             if self._data[val].upper.is_expression():
                self._data[val].upper.pprint(ostream)
             else:
                print >>ostream, str(self._data[val].upper)
          elif self._data[val]._equality:
             print >>ostream, "\t\t",
             if self._data[val].lower.is_expression():
                self._data[val].lower.pprint(ostream)
             else:
                print >>ostream, str(self._data[val].lower)
          else:
             print >>ostream, "\t\tInf"

    def display(self, prefix="", ostream=None):
        if ostream is None:
           ostream = sys.stdout
        print >>ostream, prefix+"Constraint "+self.name,":",
        print >>ostream, "  Size="+str(len(self))
        if None in self._data:
            if self._data[None].body is None:
                val = 'none'
            else:
                val = pyutilib.misc.format_io(self._data[None].body())
            print >>ostream, '%s  Value=%s' % (prefix, val)
        else:
           flag=True
           for key in self._data:
             if not self._data[key].active:
                continue
             if flag:
                print >>ostream, prefix+"        \tLower\tBody\t\tUpper"
                flag=False
             if self._data[key].lower is not None:
                lval = str(self._data[key].lower())
             else:
                lval = "-Infinity"
             val = str(self._data[key].body())
             if self._data[key].upper is not None:
                uval = str(self._data[key].upper())
             else:
                uval = "Infinity"
             print >>ostream, "%s  %s :\t%s\t%s\t%s" % (
                              prefix, str(key), lval, val, uval )
           if flag:
                print >>ostream, prefix+"  None active"



class ConstraintList(Constraint):
    """
    A constraint component that represents a list of constraints.  Constraints can
    be indexed by their index, but when they are added an index value is not specified.
    """

    alias("ConstraintList", "A list of constraints in a model.")

    def __init__(self, *args, **kwargs):
        if len(args) > 0:
            raise ValueError, "Cannot specify indices for a ConstraintList object"

        # Construct parents
        kwargs['ctype'] = kwargs.get('ctype', ConstraintList)
        self._hidden_index = Set()
        self._nconstraints = 0
        targs = [self._hidden_index]
        Constraint.__init__(self, *targs, **kwargs)
        #self._no_rule_init = kwargs.pop('noruleinit', None )
        #tmprule = kwargs.pop('rule', None )
        #tmprule = kwargs.pop('expr', tmprule )

    def construct(self, *args, **kwds):
        self._hidden_index.construct()
        Constraint.construct(self, *args, **kwds)

    def construct(self, data=None):
        #
        if __debug__:
            logger.debug("Constructing constraint %s",self.name)
        if (self._no_rule_init is not None) and (self.rule is not None):
            msg = 'WARNING: noruleinit keyword is being used in conjunction ' \
                  "with rule keyword for constraint '%s'; defaulting to "     \
                  'rule-based construction.'
            print msg % self.name
        if self.rule is None:
            if self._no_rule_init is None:
                msg = 'WARNING: No construction rule or expression '          \
                      "specified for 'constraint '%s'"
                print msg % self.name
            return
        if self._constructed:
            return
        self._constructed=True
        #
        # Local variables for code optimization
        #
        _self_rule = self.rule
        #
        while True:
            val = self._nconstraints + 1
            if __debug__:
                if logger.isEnabledFor(logging.DEBUG):
                    logger.debug("   Constructing constraint index "+str(val))
            tmp=(val, self.model)
            expr = _self_rule(*tmp)
            if expr is None or (type(expr) in (int, long, float) and expr == 0):
                break
            self.add(expr)

    def add(self, *args):
        self._nconstraints += 1
        targs = [self._nconstraints] + list(args)
        Constraint.add(self, *targs)



class SOSConstraint(ConstraintBase):
    """
    Represents an SOS-n constraint.

    Usage:
    model.C1 = SOSConstraint(
                             [...],
                             var=VAR,
                             [set=SET OR index=SET],
                             [sos=N OR level=N]
                             )
        [...] Any number of sets used to index SET
        VAR   The set of variables making up the SOS. Indexed by SET.
        SET   The set used to index VAR. SET is optionally indexed by
              the [...] sets. If SET is not specified, VAR is indexed
              over the set(s) it was defined with.
        N     This constraint is an SOS-N constraint. Defaults to 1.

    Example:

      model = AbstractModel()
      model.A = Set()
      model.B = Set(A)
      model.X = Set(B)

      model.C1 = SOSConstraint(model.A, var=model.X, set=model.B, sos=1)

    This constraint actually creates one SOS-1 constraint for each
    element of model.A (e.g., if |A| == N, there are N constraints).
    In each constraint, model.X is indexed by the elements of
    model.D[a], where 'a' is the current index of model.A.

      model = AbstractModel()
      model.A = Set()
      model.X = Var(model.A)

      model.C2 = SOSConstraint(var=model.X, sos=2)

    This produces exactly one SOS-2 constraint using all the variables
    in model.X.
    """


    alias("SOSConstraint", "SOS constraint expressions in a model.")

    def __init__(self, *args, **kwargs):
        name = kwargs.get('name', 'unknown')

        # Get the 'var' parameter
        sosVars = kwargs.pop('var', None)

        # Get the 'set' or 'index' parameters
        if 'set' in kwargs and 'index' in kwargs:
            raise TypeError, "Specify only one of 'set' and 'index' -- " \
                  "they are equivalent parameters"
        sosSet = kwargs.pop('set', None)
        sosSet = kwargs.pop('index', sosSet)

        # Get the 'sos' or 'level' parameters
        if 'sos' in kwargs and 'index' in kwargs:
            raise TypeError, "Specify only one of 'sos' and 'level' -- " \
                  "they are equivalent parameters"
        sosLevel = kwargs.pop('sos', None)
        sosLevel = kwargs.pop('level', sosLevel)

        # Make sure sosLevel has been set
        if sosLevel is None:
            raise TypeError, "SOSConstraint() requires that either the " \
                  "'sos' or 'level' keyword arguments be set to indicate " \
                  "the type of SOS."

        # Make sure we have a variable
        if sosVars is None:
            raise TypeError, "SOSConstraint() requires the 'var' keyword " \
                  "be specified"

        # Find the default sets for sosVars if sosSets is None
        if sosSet is None:
            sosSet = sosVars.index()

        # Construct parents
        kwargs['ctype'] = kwargs.get('ctype', SOSConstraint)
        ConstraintBase.__init__(self, *args, **kwargs)

        # Set member attributes
        self._sosVars = sosVars
        self._sosSet = sosSet
        self._sosLevel = sosLevel

        # TODO figure out why exactly the code expects this to be defined
        # likely due to Numericvalue
        self.domain = None

        # TODO should variables be ordered?

    def construct(self, *args, **kwds):
        """
        A quick hack to call add after data has been loaded. construct
        doesn't actually need to do anything.
        """
        self.add()

    def add(self, *args):
        """
        Mimics Constraint.add, but is only used to alert preprocessors
        to the existence of the SOS variables.

        Ignores all arguments passed to it.
        """

        # self._data is a ConstraintData object, which has a member
        # .body.  .body needs to have a 3-tuple of expressions
        # containing the variables that are referenced by this
        # constraint. We only use one index, None. I have no
        # particular reason to use None, it just seems consistent with
        # what Constraint objects do.

        # For simplicity, we sum over the variables.

        vars = self.sos_vars()

        expr = sum(vars[i] for i in vars._index)

        self._data[None] = ConstraintData(name=create_name(self.name, None), con=self)
        self._data[None].body = expr

    def sos_level(self):
        """ Return n, where this class is an SOS-n constraint """
        return self._sosLevel

    def sos_vars(self):
        """ Return the variables in the SOS """
        return self._sosVars

    def sos_set(self):
        """ Return the set used to index the variables """
        return self._sosSet

    def sos_set_set(self):
        """ Return the sets used to index the sets indexing the variables """
        return self._index

    def pprint(self, ostream=None):
        if ostream is None:
           ostream = sys.stdout
        print >>ostream, "  ",self.name,":",
        print >>ostream, "Type="+str(self._sosLevel)
        print >>ostream, "\tVariable: "+self._sosVars.name
        print >>ostream, "\tIndices: ",
        for i in self._sosSet.value:
           print >>ostream, str(i)+" ",
        print >>ostream, ""



#
# Test the Pyomo command-line interface
#

import os
import sys
from os.path import abspath, dirname
currdir = dirname(abspath(__file__))+os.sep
datadir = os.path.normpath(currdir+'../../../../examples/pyomo/p-median/')+os.sep

from coopr.pyomo import *
import pyutilib.th as unittest
import coopr.pyomo.scripting.convert as convert
import re
import glob

class Test(unittest.TestCase):

    def setUp(self):
        self.cwd = os.getcwd()
        os.chdir(currdir)

    def tearDown(self):
        os.chdir(self.cwd)

    def test1(self):
        convert.pyomo2nl(['--save-model',currdir+'test1.nl','pmedian.py','pmedian.dat'])
        if not os.path.exists(currdir+'test1.nl'):
            raise ValueError, "Missing file test1.nl generated in test1"
        os.remove(currdir+'test1.nl')

    def test2(self):
        convert.pyomo2lp(['--save-model',currdir+'test2.lp','pmedian.py','pmedian.dat'])
        if not os.path.exists(currdir+'test2.lp'):
            raise ValueError, "Missing file test2.lp generated in test2"
        os.remove(currdir+'test2.lp')

Test = unittest.category('performance')(Test)

@unittest.nottest
def nl_test(self, suite, name):
    fname = currdir+name+'.nl'
    root = name.split('_')[1]
    #print >>sys.stderr, fname
    convert.pyomo2nl(['--save-model',fname]+self._options[suite,name]+[datadir+root+'.dat'])
    if not os.path.exists(fname):
        raise ValueError, "Missing file %s generated in test2" % fname
    os.remove(fname)

@unittest.nottest
def lp_test(self, suite, name):
    fname = currdir+name+'.lp'
    root = name.split('_')[1]
    #print >>sys.stderr, fname
    convert.pyomo2lp(['--save-model',fname]+self._options[suite,name]+[datadir+root+'.dat'])
    if not os.path.exists(fname):
        raise ValueError, "Missing file %s generated in test2" % fname
    os.remove(fname)


for i in range(8):
    name = 'test'+str(i+1)
    #
    Test.add_fn_test(Test, fn=nl_test, name='nl_pmedian.'+name, options=[datadir+'pmedian.py'])
    Test.add_fn_test(Test, fn=nl_test, name='nl-O_pmedian.'+name, options=['--disable-gc', datadir+'pmedian.py'] )
    #
    Test.add_fn_test(Test, fn=lp_test, name='lp_pmedian.'+name, options=[datadir+'pmedian.py'])
    Test.add_fn_test(Test, fn=lp_test, name='lp-O_pmedian.'+name, options=['--disable-gc', datadir+'pmedian.py'] )
    #
    #Test.add_fn_test(Test, fn=nl_test, name='nl_pmedian_v2.'+name, options=[datadir+'pmedianv2.py'])
    #Test.add_fn_test(Test, fn=nl_test, name='nl-O_pmedian_v2.'+name, options=['--disable-gc', datadir+'pmedianv2.py'] )
    #
    #Test.add_fn_test(Test, fn=lp_test, name='lp_pmedian_v2.'+name, options=[datadir+'pmedianv2.py'])
    #Test.add_fn_test(Test, fn=lp_test, name='lp-O_pmedian_v2.'+name, options=['--disable-gc', datadir+'pmedianv2.py'] )


if __name__ == "__main__":
   unittest.main()



# Create the variables and sets
model.M = RangeSet(0,10)
model.N = Set(initialize=[1,3,5,7,9])
model.x = Var(model.M)

# Create SOS1 constraints for the odd-indexed x variables
model.sos1 = SOSConstraint(var=model.x, set=model.M, level=1)



# Create the variables and sets
model.M = RangeSet(0,2)

def nRule(m, model):
    if m == 0:
        return [0,1,2]
    elif m == 1:
        return [3,4,5]
    else:
        return [6,7,8]

model.N = Set(model.M, initialize=nRule)
model.W = RangeSet(0,8)
model.x = Var(model.W)

# Make 3 SOS1 constraints
model.sos1 = SOSConstraint(model.M, var=model.x, set=model.N,
                           level=1)



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the FAST README.txt file.
#  _________________________________________________________________________


import pyutilib.component.core
from coopr.pyomo.base import expr, Constraint, Objective, SOSConstraint
from coopr.pyomo.base.var import  _VarValue, VarStatus, _VarArray, _VarBase, Var
from coopr.pyomo.base.numvalue import NumericConstant
from coopr.pyomo.base import IPyomoPresolver, IPyomoPresolveAction


class IdentifyVariablesPresolver(pyutilib.component.core.SingletonPlugin):
    """
    This plugin processes a Pyomo Model instance to define
    the order of variables and to categorize constraints.
    """

    pyutilib.component.core.implements(IPyomoPresolveAction)

    def __init__(self, **kwds):
        kwds['name'] = "identify_variables"
        pyutilib.component.core.Plugin.__init__(self, **kwds)

    def rank(self):
        return 1

    #
    # Scan through a linearly encoded expression body to identify variables.
    #
    def _identify_variables_in_linear_encoding(self, encoding, var_map):

       # recall that "encoding" is a tuple: element 0 is the constant, element 1 is a list of (coefficient, _VarValue) pairs.
       for coefficient, var in encoding[1]:

          if var.fixed: # we know this is a _VarValue, by construction - do the direct query to save time.
             continue
          else:
             if var.id in var_map:
                continue
             var.status = VarStatus.used
             var.id = self.var_count
             var_map[self.var_count] = var
             self.var_count += 1

    #
    # Recursively work through an expression (tree) to identify variables.
    #
    def _identify_variables_in_expression(self, exp, var_map):

        try:
            # It is possible (likely) that the expression is not
            # completely valid.  Check if that is the case, and return a
            # useful message to the user.
            is_expr = exp.is_expression()
        except AttributeError, e:
            # Likely caused by a constraint or an objective failing
            # to return a value. 
            if isinstance(exp, _VarArray):
                msg = "Have you properly indexed '%s'?" % str(exp)
            else:
                msg = "Do all constraints and objectives return a value?"
            logging.getLogger('coopr.pyomo').error(
                "identifying variables in expression failed: %s\n%s\ntype=", 
                str(e), msg, str(type(exp)) )
            raise 
                
        #
        # Expression
        #
        if is_expr:
            #
            # Product Expressions store stuff differently
            #
            if exp.__class__ is expr._ProductExpression:
                for arg in exp._numerator:
                    self._identify_variables_in_expression(arg, var_map)
                for arg in exp._denominator:
                    self._identify_variables_in_expression(arg, var_map)
            else:
                for arg in exp._args:
                    self._identify_variables_in_expression(arg, var_map)
        #
        # Fixed Values (params, constants, fixed variables)
        #
        elif exp.is_constant():
            return
        #
        # Variable Value
        #
        elif isinstance(exp,_VarValue):
            if exp.id in var_map:
                return
            exp.status = VarStatus.used
            exp.id = self.var_count
            var_map[self.var_count] = exp
            self.var_count += 1
        #
        # Variable Base
        #
        elif isinstance(exp,_VarBase):
            tmp = exp.as_numeric()
            if tmp is exp:
                raise ValueError("Variable %s appears in an expression without "
                                 "an index, but this variable is an indexed "
                                 "value.", exp.name)
            if tmp.id in var_map:
                return
            exp.status = VarStatus.used
            tmp.id = self.var_count
            var_map[self.var_count] = tmp
            self.var_count += 1            
        #
        # If not a constant, then this is an error
        #
        else:
            raise ValueError, "Unexpected expression type in identify_variables: " + str(type(exp))+" "+str(exp)

    #
    # this pre-processor computes a number of attributes of model components, in 
    # addition to models themselves. for VarValue, ConstraintData, and ObjectiveData,
    # the routine assigns the "id" attribute. in addition, for VarValues, the routine
    # sets the "status" attribute - to used or unused, depending on whether it appears
    # in an active constraint/objective. on the Model object, the routine populates
    # the "statistics" (variable/constraint/objective counts) attribute, and builds 
    # the _var, _con, and _obj dictionaries of a Model, which map the id attributes
    # to the respective VarValue, ConstraintData, or ObjectiveData object.
    #
    # NOTE: if variables are fixed, then this preprocessor will flag them as unused
    #       and consequently not create entries for them in the model _var map. 
    #

    def preprocess(self,model):
        """
        The main routine to perform the preprocess
        """

        active_variables = model.active_components(Var)
        active_constraints = model.active_components(Constraint)
        active_objectives = model.active_components(Objective)

        active_constraints.update(model.active_components(SOSConstraint))

        self.var_count=0
        self.constraint_count=0
        self.objective_count=0
        
        model.statistics.number_of_binary_variables = 0
        model.statistics.number_of_integer_variables = 0
        model.statistics.number_of_continuous_variables = 0

        #
        # Reset the _var map. Failing to reset can cause problems in cloned
        # instances.
        #
        _var_map = model._var = {}
        
        #
        # Until proven otherwise, all variables are unused.
        #
        for var in active_variables.itervalues():
            for var_value in var._varval.itervalues():
                var_value.status = VarStatus.unused
                var_value.id = -1

        #
        # Call the appropriate identify_variables to find the variables that 
        # are actively used in the objective and constraints.
        #
        for name, objective in active_objectives.iteritems():
            for index, objective_data in objective._data.iteritems():
                if not objective_data.active:
                    continue
                try:
                    self._identify_variables_in_expression(objective_data.expr, _var_map)
                except ValueError, err:
                    raise ValueError, "Problem processing objective %s (index %s): %s" % \
                        (str(name), str(index), str(err))

                objective_data.id = self.objective_count
                model._obj[self.objective_count] = objective_data
                self.objective_count += 1

        for name, constraint in active_constraints.iteritems():
            for index, constraint_data in constraint._data.iteritems():
                if not constraint_data.active:
                    continue
                try:
                    if constraint_data.lin_body is not None:
                       self._identify_variables_in_linear_encoding(constraint_data.lin_body, _var_map)
                    else:
                       self._identify_variables_in_expression(constraint_data.body, _var_map)
                except ValueError, err:
                    raise ValueError, "Problem processing constraint %s (index %s): %s" % \
                        (str(name), str(index), str(err))
                constraint_data.id = self.constraint_count
                model._con[self.constraint_count] = constraint_data
                self.constraint_count += 1

        #
        # count the number of variables in each type category.
        # we don't do this within the _identify_variables_in_expression function
        # because the variable type is owned by the "parent"
        # variable, and the interrogation cost for variable type
        # is comparatively expensive - we don't want to do it for
        # each variable value.
        #
        for var in active_variables.itervalues():
            model.statistics.number_of_binary_variables += len(var.binary_keys())
            model.statistics.number_of_integer_variables += len(var.integer_keys())
            model.statistics.number_of_continuous_variables += len(var.continuous_keys())

        #
        # assign the model statistics
        #

        model.statistics.number_of_variables = self.var_count
        model.statistics.number_of_constraints = self.constraint_count
        model.statistics.number_of_objectives = self.objective_count

        return model

